@@ -2,9 +2,13 @@
 #include "logging_private.h"
 #include "ControlSocket.h"
 #include <idna.h>
+extern "C" {
+#include <idn-free.h>
+}
 #include "directorycache.h"
 #include "servercapabilities.h"
 #include "local_filesys.h"
+#include "local_path.h"
 #include <errno.h>
 #include "proxy.h"
 
@@ -181,6 +185,7 @@ int CControlSocket::ResetOperation(int nErrorCode)
 
 int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 {
+	LogMessage(Debug_Debug, _T("CControlSocket::DoClose(%d)"), nErrorCode);
 	if (m_closed)
 	{
 		wxASSERT(!m_pCurOpData);
@@ -219,7 +224,7 @@ wxString CControlSocket::ConvertDomainName(wxString domain)
 	delete [] utf8;
 
 	wxString result = wxConvCurrent->cMB2WX(output);
-	free(output);
+	idn_free(output);
 	return result;
 }
 
@@ -769,8 +774,19 @@ void CControlSocket::UnlockCache()
 	m_lockInfoList.erase(iter);
 
 	// Find other instance waiting for the lock
+	if (!m_pCurrentServer)
+	{
+		LogMessage(Debug_Warning, _T("UnlockCache called with !m_pCurrentServer"));
+		return;
+	}
 	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != m_lockInfoList.end(); iter++)
 	{
+		if (!iter->pControlSocket->m_pCurrentServer)
+		{
+			LogMessage(Debug_Warning, _T("UnlockCache found other instance with !m_pCurrentServer"));
+			continue;
+		}
+
 		if (*m_pCurrentServer != *iter->pControlSocket->m_pCurrentServer)
 			continue;
 
@@ -929,6 +945,7 @@ bool CRealControlSocket::Send(const char *buffer, int len)
 				DoClose();
 				return false;
 			}
+			written = 0;
 		}
 
 		if (written)
@@ -1314,3 +1331,48 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 
 	return true;
 }
+
+void CControlSocket::CreateLocalDir(const wxString &local_file)
+{
+	wxString file;
+	CLocalPath local_path(local_file, &file);
+	if (local_path.empty() || !local_path.HasParent())
+		return;
+
+	// Only go back as far as needed. On comparison, wxWidgets'
+	// wxFileName::Mkdir always starts at the root.
+	std::list<wxString> segments;
+	while (!local_path.Exists() && local_path.HasParent())
+	{
+		wxString segment;
+		local_path.MakeParent(&segment);
+		segments.push_front(segment);
+	}
+
+	CLocalPath last_successful;
+	for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); iter++)
+	{
+		local_path.AddSegment(*iter);
+
+#ifdef __WXMSW__
+		BOOL res = CreateDirectory(local_path.GetPath(), 0);
+		if (!res && GetLastError() != ERROR_ALREADY_EXISTS)
+			break;
+#else
+		const wxCharBuffer s = local_path.GetPath().fn_str();
+
+		int res = mkdir(s, 0777);
+		if (res && errno != EEXIST)
+			break;
+#endif
+		last_successful = local_path;
+	}
+
+	if (last_successful.empty())
+		return;
+
+	// Send out notification
+	CLocalDirCreatedNotification *n = new CLocalDirCreatedNotification;
+	n->dir = last_successful;
+	m_pEngine->AddNotification(n);
+}
