@@ -27,13 +27,13 @@ protected:
 	virtual int ResetOperation(int nErrorCode);
 
 	virtual int Connect(const CServer &server);
-	virtual int List(CServerPath path = CServerPath(), wxString subDir = _T(""), bool refresh = false);
+	virtual int List(CServerPath path = CServerPath(), wxString subDir = _T(""), int flags = 0);
 	int ListParseResponse();
 	int ListSubcommandResult(int prevResult);
 	int ListSend();
 	int ListCheckTimezoneDetection(CDirectoryListing& listing);
 
-	int ChangeDir(CServerPath path = CServerPath(), wxString subDir = _T(""));
+	int ChangeDir(CServerPath path = CServerPath(), wxString subDir = _T(""), bool link_discovery = false);
 	int ChangeDirParseResponse();
 	int ChangeDirSubcommandResult(int prevResult);
 	int ChangeDirSend();
