@@ -175,7 +175,6 @@ bool IsRoutableAddress(const wxString& address, int family)
 	if (family == AF_INET6)
 	{
 		wxString long_address = GetIPV6LongForm(address);
-		printf("\n--- %s ---\n", (const char*)long_address.mb_str());
 		if (long_address.empty())
 			return false;
 		if (long_address[0] == '0')
@@ -364,3 +363,13 @@ int GetRandomNumber(int low, int high)
 		return (int)((r % (range + 1)) + low);
 	}
 }
+
+void MakeLowerAscii(wxString& str)
+{
+	for (size_t i = 0; i < str.Len(); i++)
+	{
+		wxChar& c = str[i];
+		if (c >= 'A' && c <= 'Z')
+			c += 32;
+	}
+}
