@@ -1410,14 +1410,14 @@ int CSftpControlSocket::ChangeDirParseResponse(bool successful, const wxString&
 
 int CSftpControlSocket::ChangeDirSubcommandResult(int WXUNUSED(prevResult))
 {
-	LogMessage(Debug_Verbose, _("CSftpControlSocket::ChangeDirSubcommandResult()"));
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::ChangeDirSubcommandResult()"));
 
 	return SendNextCommand();
 }
 
 int CSftpControlSocket::ChangeDirSend()
 {
-	LogMessage(Debug_Verbose, _("CSftpControlSocket::ChangeDirSend()"));
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::ChangeDirSend()"));
 
 	if (!m_pCurOpData)
 	{
@@ -1777,12 +1777,8 @@ int CSftpControlSocket::FileTransferSend()
 			cmd = _T("re");
 		if (pData->download)
 		{
-			// Create local directory
 			if (!pData->resume)
-			{
-				wxFileName fn(pData->localFile);
-				wxFileName::Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
-			}
+				CreateLocalDir(pData->localFile);
 
 			InitTransferStatus(pData->remoteFileSize, pData->resume ? pData->localFileSize : 0, false);
 			cmd += _T("get ");
