@@ -21,8 +21,12 @@
   #include <sys/socket.h>
   #include <netdb.h>
   #include <fcntl.h>
+  #include <arpa/inet.h>
   #include <netinet/in.h>
   #include <netinet/tcp.h>
+  #if !defined(MSG_NOSIGNAL) && !defined(SO_NOSIGPIPE)
+    #include <signal.h>
+  #endif
 #endif
 
 // Fixups needed on FreeBSD
@@ -93,8 +97,6 @@ static CGetAddrinfoLoader addrinfo_loader;
 #define WAIT_CLOSE	 0x10
 #define WAIT_EVENTCOUNT 5
 
-const wxEventType fzEVT_SOCKET = wxNewEventType();
-
 class CSocketThread;
 static std::list<CSocketThread*> waiting_socket_threads;
 
@@ -159,21 +161,153 @@ public:
 };
 #endif
 
-CSocketEvent::CSocketEvent(int id, enum EventType type, wxString data)
-	: wxEvent(id, fzEVT_SOCKET), m_type(type), m_data(data), m_error(0)
+CSocketEventDispatcher CSocketEventDispatcher::m_dispatcher;
+
+CSocketEventDispatcher::CSocketEventDispatcher()
 {
 }
 
-CSocketEvent::CSocketEvent(int id, enum EventType type, int error /*=0*/)
-	: wxEvent(id, fzEVT_SOCKET), m_type(type), m_error(error)
+CSocketEventDispatcher::~CSocketEventDispatcher()
 {
+	for (std::list<CSocketEvent*>::iterator iter = m_pending_events.begin(); iter != m_pending_events.end(); iter++)
+		delete *iter;
 }
 
-wxEvent* CSocketEvent::Clone() const
+CSocketEventDispatcher& CSocketEventDispatcher::Get()
 {
-	CSocketEvent* pClone = new CSocketEvent(GetId(), GetType(), m_error);
-	pClone->m_data = m_data;
-	return pClone;
+	return m_dispatcher;
+}
+
+void CSocketEventDispatcher::SendEvent(CSocketEvent* evt)
+{
+	wxCriticalSectionLocker lock(m_sync);
+
+	const bool send = m_pending_events.empty() && !m_inside_loop;
+
+	m_pending_events.push_back(evt);
+
+	if (!send)
+		return;
+
+	wxCommandEvent tmp(0, 0);
+	AddPendingEvent(tmp);
+}
+
+void CSocketEventDispatcher::RemovePending(const CSocketEventHandler* pHandler)
+{
+	wxCriticalSectionLocker lock(m_sync);
+
+	std::list<CSocketEvent*> keep;
+	for (std::list<CSocketEvent*>::iterator iter = m_pending_events.begin(); iter != m_pending_events.end(); iter++)
+	{
+		if ((*iter)->GetSocketEventHandler() != pHandler)
+		{
+			keep.push_back(*iter);
+			continue;
+		}
+
+		delete *iter;
+	}
+	m_pending_events.swap(keep);
+}
+
+void CSocketEventDispatcher::RemovePending(const CSocketEventSource* pSource)
+{
+	wxCriticalSectionLocker lock(m_sync);
+
+	std::list<CSocketEvent*> keep;
+	for (std::list<CSocketEvent*>::iterator iter = m_pending_events.begin(); iter != m_pending_events.end(); iter++)
+	{
+		if ((*iter)->GetSocketEventSource() != pSource)
+		{
+			keep.push_back(*iter);
+			continue;
+		}
+
+		delete *iter;
+	}
+	m_pending_events.swap(keep);
+}
+
+void CSocketEventDispatcher::UpdatePending(const CSocketEventHandler* pOldHandler, const CSocketEventSource* pOldSource, CSocketEventHandler* pNewHandler, CSocketEventSource* pNewSource)
+{
+	wxCriticalSectionLocker lock(m_sync);
+
+	for (std::list<CSocketEvent*>::iterator iter = m_pending_events.begin(); iter != m_pending_events.end(); iter++)
+	{
+		CSocketEvent* evt = *iter;
+		if (evt->GetSocketEventSource() != pOldSource || evt->GetSocketEventHandler() != pOldHandler)
+			continue;
+
+		evt->m_pSource = pNewSource;
+		evt->m_pSocketEventHandler = pNewHandler;
+	}
+}
+
+bool CSocketEventDispatcher::ProcessEvent(wxEvent& event)
+{
+	m_sync.Enter();
+	m_inside_loop = true;
+	while (!m_pending_events.empty())
+	{
+		CSocketEvent *evt = m_pending_events.front();
+		m_pending_events.pop_front();
+
+		m_sync.Leave();
+
+		CSocketEventHandler* pHandler = evt->GetSocketEventHandler();
+		wxASSERT(pHandler);
+		pHandler->OnSocketEvent(*evt);
+
+		delete evt;
+
+		m_sync.Enter();
+	}
+	m_inside_loop = false;
+	m_sync.Leave();
+
+	return true;
+}
+
+CSocketEvent::CSocketEvent(CSocketEventHandler* pSocketEventHandler, CSocketEventSource* pSource, enum EventType type, const wxChar* data)
+	: m_pSource(pSource), m_type(type), m_error(0), m_pSocketEventHandler(pSocketEventHandler)
+{
+	if (data)
+	{
+		m_data = new wxChar[wxStrlen(data) + 1];
+		wxStrcpy(m_data, data);
+	}
+	else
+		m_data = 0;
+}
+
+CSocketEvent::CSocketEvent(CSocketEventHandler* pSocketEventHandler, CSocketEventSource* pSource, enum EventType type, int error /*=0*/)
+	: m_pSource(pSource), m_type(type), m_error(error), m_pSocketEventHandler(pSocketEventHandler)
+{
+	m_data = 0;
+}
+
+CSocketEvent::~CSocketEvent()
+{
+	delete [] m_data;
+}
+
+wxString CSocketEvent::GetData() const
+{
+	if (!m_data)
+		return wxEmptyString;
+
+	return m_data;
+}
+
+CSocketEventHandler::~CSocketEventHandler()
+{
+	CSocketEventDispatcher::Get().RemovePending(this);
+}
+
+CSocketEventSource::~CSocketEventSource()
+{
+	CSocketEventDispatcher::Get().RemovePending(this);
 }
 
 class CSocketThread : protected wxThreadEx
@@ -248,6 +382,12 @@ public:
 			delete [] m_pPort;
 
 		const wxWX2MBbuf buf = m_pSocket->m_host.mb_str();
+		if (!buf)
+		{
+			m_pHost = 0;
+			m_pPort = 0;
+			return EINVAL;
+		}
 		m_pHost = new char[strlen(buf) + 1];
 		strcpy(m_pHost, buf);
 
@@ -329,26 +469,15 @@ public:
 
 protected:
 
-	bool SendEvent(CSocketEvent& evt)
-	{
-		if (!m_pSocket || !m_pSocket->m_pEvtHandler)
-			return false;
-
-		m_pSocket->m_pEvtHandler->AddPendingEvent(evt);
-
-		return true;
-	}
-
 	int TryConnectHost(struct addrinfo *addr)
 	{
-		CSocketEvent evt(m_pSocket->m_id, CSocketEvent::hostaddress, CSocket::AddressToString(addr->ai_addr, addr->ai_addrlen));
-		SendEvent(evt);
+		if (m_pSocket->m_pEvtHandler)
+		{
+			CSocketEvent *evt = new CSocketEvent(m_pSocket->m_pEvtHandler, m_pSocket, CSocketEvent::hostaddress, CSocket::AddressToString(addr->ai_addr, addr->ai_addrlen));
+			CSocketEventDispatcher::Get().SendEvent(evt);
+		}
 
 		int fd = socket(addr->ai_family, addr->ai_socktype, addr->ai_protocol);
-		CSocket::DoSetFlags(fd, m_pSocket->m_flags, m_pSocket->m_flags);
-
-		CSocket::DoSetBufferSizes(fd, m_pSocket->m_buffer_sizes[0], m_pSocket->m_buffer_sizes[1]);
-
 		if (fd == -1)
 		{
 #ifdef __WXMSW__
@@ -356,12 +485,23 @@ protected:
 #else
 			int res = errno;
 #endif
-			CSocketEvent evt(m_pSocket->m_id, addr->ai_next ? CSocketEvent::connection_next : CSocketEvent::connection, res);
-			SendEvent(evt);
+			if (m_pSocket->m_pEvtHandler)
+			{
+				CSocketEvent *evt = new CSocketEvent(m_pSocket->GetEventHandler(), m_pSocket, addr->ai_next ? CSocketEvent::connection_next : CSocketEvent::connection, res);
+				CSocketEventDispatcher::Get().SendEvent(evt);
+			}
 
 			return 0;
 		}
+		
+#if defined(SO_NOSIGPIPE) && !defined(MSG_NOSIGNAL)
+		// We do not want SIGPIPE if writing to socket.
+		const int value = 1;
+		setsockopt(fd, SOL_SOCKET, SO_NOSIGPIPE, &value, sizeof(int));
+#endif
 
+		CSocket::DoSetFlags(fd, m_pSocket->m_flags, m_pSocket->m_flags);
+		CSocket::DoSetBufferSizes(fd, m_pSocket->m_buffer_sizes[0], m_pSocket->m_buffer_sizes[1]);
 		CSocket::SetNonblocking(fd);
 
 		int res = connect(fd, addr->ai_addr, addr->ai_addrlen);
@@ -410,8 +550,11 @@ protected:
 
 		if (res)
 		{
-			CSocketEvent evt(m_pSocket->m_id, addr->ai_next ? CSocketEvent::connection_next : CSocketEvent::connection, res);
-			SendEvent(evt);
+			if (m_pSocket->m_pEvtHandler)
+			{
+				CSocketEvent *evt = new CSocketEvent(m_pSocket->GetEventHandler(), m_pSocket, addr->ai_next ? CSocketEvent::connection_next : CSocketEvent::connection, res);
+				CSocketEventDispatcher::Get().SendEvent(evt);
+			}
 
 			m_pSocket->m_fd = -1;
 #ifdef __WXMSW__
@@ -425,8 +568,11 @@ protected:
 			m_pSocket->m_fd = fd;
 			m_pSocket->m_state = CSocket::connected;
 
-			CSocketEvent evt(m_pSocket->m_id, CSocketEvent::connection, 0);
-			SendEvent(evt);
+			if (m_pSocket->m_pEvtHandler)
+			{
+				CSocketEvent *evt = new CSocketEvent(m_pSocket->GetEventHandler(), m_pSocket, CSocketEvent::connection, 0);
+				CSocketEventDispatcher::Get().SendEvent(evt);
+			}
 
 			// We're now interested in all the other nice events
 			m_waiting |= WAIT_READ | WAIT_WRITE;
@@ -481,9 +627,12 @@ protected:
 
 			if (!h)
 			{
-				int res = ConvertMSWErrorCode(WSAGetLastError());
-				CSocketEvent evt(m_pSocket->m_id, CSocketEvent::connection, res);
-				SendEvent(evt);
+				if (m_pSocket->m_pEvtHandler)
+				{
+					int res = ConvertMSWErrorCode(WSAGetLastError());
+					CSocketEvent *evt = new CSocketEvent(m_pSocket->GetEventHandler(), m_pSocket, CSocketEvent::connection, res);
+					CSocketEventDispatcher::Get().SendEvent(evt);
+				}
 				m_pSocket->m_state = CSocket::closed;
 				return false;
 			}
@@ -504,7 +653,14 @@ protected:
 				return true;
 
 			if (m_pSocket)
+			{
+				if (!res && m_pSocket->m_pEvtHandler)
+				{
+					CSocketEvent *evt = new CSocketEvent(m_pSocket->GetEventHandler(), m_pSocket, CSocketEvent::connection, ECONNABORTED);
+					CSocketEventDispatcher::Get().SendEvent(evt);
+				}
 				m_pSocket->m_state = CSocket::closed;
+			}
 			return false;
 		}
 #endif
@@ -528,8 +684,11 @@ protected:
 			res = ConvertMSWErrorCode(res);
 #endif
 
-			CSocketEvent evt(m_pSocket->m_id, CSocketEvent::connection, res);
-			SendEvent(evt);
+			if (m_pSocket->m_pEvtHandler)
+			{
+				CSocketEvent *evt = new CSocketEvent(m_pSocket->GetEventHandler(), m_pSocket, CSocketEvent::connection, res);
+				CSocketEventDispatcher::Get().SendEvent(evt);
+			}
 			m_pSocket->m_state = CSocket::closed;
 
 			return false;
@@ -552,6 +711,12 @@ protected:
 			}
 		}
 		freeaddrinfo(addressList);
+
+		if (m_pSocket->m_pEvtHandler)
+		{
+			CSocketEvent *evt = new CSocketEvent(m_pSocket->GetEventHandler(), m_pSocket, CSocketEvent::connection, ECONNABORTED);
+			CSocketEventDispatcher::Get().SendEvent(evt);
+		}
 		m_pSocket->m_state = CSocket::closed;
 
 		return false;
@@ -746,30 +911,30 @@ protected:
 
 	void SendEvents()
 	{
-		if (!m_pSocket->m_pEvtHandler)
+		if (!m_pSocket || !m_pSocket->m_pEvtHandler)
 			return;
 		if (m_triggered & WAIT_READ)
 		{
-			CSocketEvent evt(m_pSocket->m_id, CSocketEvent::read, m_triggered_errors[1]);
-			m_pSocket->m_pEvtHandler->AddPendingEvent(evt);
+			CSocketEvent *evt = new CSocketEvent(m_pSocket->m_pEvtHandler, m_pSocket, CSocketEvent::read, m_triggered_errors[1]);
+			CSocketEventDispatcher::Get().SendEvent(evt);
 			m_triggered &= ~WAIT_READ;
 		}
 		if (m_triggered & WAIT_WRITE)
 		{
-			CSocketEvent evt(m_pSocket->m_id, CSocketEvent::write, m_triggered_errors[2]);
-			m_pSocket->m_pEvtHandler->AddPendingEvent(evt);
+			CSocketEvent *evt = new CSocketEvent(m_pSocket->m_pEvtHandler, m_pSocket, CSocketEvent::write, m_triggered_errors[2]);
+			CSocketEventDispatcher::Get().SendEvent(evt);
 			m_triggered &= ~WAIT_WRITE;
 		}
 		if (m_triggered & WAIT_ACCEPT)
 		{
-			CSocketEvent evt(m_pSocket->m_id, CSocketEvent::connection, m_triggered_errors[3]);
-			m_pSocket->m_pEvtHandler->AddPendingEvent(evt);
+			CSocketEvent *evt = new CSocketEvent(m_pSocket->m_pEvtHandler, m_pSocket, CSocketEvent::connection, m_triggered_errors[3]);
+			CSocketEventDispatcher::Get().SendEvent(evt);
 			m_triggered &= ~WAIT_ACCEPT;
 		}
 		if (m_triggered & WAIT_CLOSE)
 		{
-			CSocketEvent evt(m_pSocket->m_id, CSocketEvent::close, m_triggered_errors[4]);
-			m_pSocket->m_pEvtHandler->AddPendingEvent(evt);
+			CSocketEvent *evt = new CSocketEvent(m_pSocket->m_pEvtHandler, m_pSocket, CSocketEvent::close, m_triggered_errors[4]);
+			CSocketEventDispatcher::Get().SendEvent(evt);
 			m_triggered &= ~WAIT_CLOSE;
 		}
 	}
@@ -892,8 +1057,8 @@ protected:
 	bool m_threadwait;
 };
 
-CSocket::CSocket(wxEvtHandler* pEvtHandler, int id)
-	: m_pEvtHandler(pEvtHandler), m_id(id)
+CSocket::CSocket(CSocketEventHandler* pEvtHandler)
+	: m_pEvtHandler(pEvtHandler)
 {
 #ifdef ERRORCODETEST
 	CErrorCodeTest test;
@@ -959,8 +1124,6 @@ int CSocket::Connect(wxString host, unsigned int port)
 	if (port < 1 || port > 65535)
 		return EINVAL;
 
-	m_state = connecting;
-
 	if (m_pSocketThread && m_pSocketThread->m_started)
 	{
 		m_pSocketThread->m_sync.Lock();
@@ -990,32 +1153,49 @@ int CSocket::Connect(wxString host, unsigned int port)
 		m_pSocketThread->SetSocket(this);
 	}
 
+	m_state = connecting;
+
 	m_host = host;
 	m_port = port;
-	m_pSocketThread->Connect();
+	int res = m_pSocketThread->Connect();
+	if (res)
+	{
+		m_state = none;
+		delete m_pSocketThread;
+		m_pSocketThread = 0;
+		return res;
+	}
 
 	return EINPROGRESS;
 }
 
-void CSocket::SetEventHandler(wxEvtHandler* pEvtHandler, int id)
+void CSocket::SetEventHandler(CSocketEventHandler* pEvtHandler)
 {
 	if (m_pSocketThread)
 		m_pSocketThread->m_sync.Lock();
 
+	if (!pEvtHandler)
+	{
+		CSocketEventDispatcher::Get().RemovePending(m_pEvtHandler);
+	}
+	else
+	{
+		if (m_pEvtHandler)
+			CSocketEventDispatcher::Get().UpdatePending(m_pEvtHandler, this, pEvtHandler, this);
+	}
 #ifdef __WXMSW__
-	if (pEvtHandler && m_state == closing && m_pSocketThread)
+	if (pEvtHandler && !m_pEvtHandler && m_state == closing && m_pSocketThread)
 	{
 		// After getting FD_CLOSE, no further events are recorded, so send
 		// it out to new handler manually
-		CSocketEvent evt(id, CSocketEvent::close, m_pSocketThread->m_triggered_errors[4]);
-		pEvtHandler->AddPendingEvent(evt);
+		CSocketEvent *evt = new CSocketEvent(pEvtHandler, this, CSocketEvent::close, m_pSocketThread->m_triggered_errors[4]);
+		CSocketEventDispatcher::Get().SendEvent(evt);
 	}
 #else
 	wxASSERT(!pEvtHandler || m_state != closing);
 #endif
 
 	m_pEvtHandler = pEvtHandler;
-	m_id = id;
 
 	if (m_pSocketThread)
 	{
@@ -1028,12 +1208,12 @@ void CSocket::SetEventHandler(wxEvtHandler* pEvtHandler, int id)
 
 			if (!(m_pSocketThread->m_waiting & WAIT_WRITE))
 			{
-				CSocketEvent evt(id, CSocketEvent::write, 0);
-				pEvtHandler->AddPendingEvent(evt);
+				CSocketEvent *evt = new CSocketEvent(pEvtHandler, this, CSocketEvent::write, 0);
+				CSocketEventDispatcher::Get().SendEvent(evt);
 			}
 
-			CSocketEvent evt(id, CSocketEvent::read, 0);
-			pEvtHandler->AddPendingEvent(evt);
+			CSocketEvent *evt = new CSocketEvent(pEvtHandler, this, CSocketEvent::read, 0);
+			CSocketEventDispatcher::Get().SendEvent(evt);
 			if (m_pSocketThread->m_waiting & WAIT_WRITE)
 			{
 				m_pSocketThread->m_waiting &= ~WAIT_READ;
@@ -1177,6 +1357,9 @@ int CSocket::Close()
 #endif
 		if (m_pSocketThread)
 			m_pSocketThread->m_sync.Unlock();
+
+		if (m_pEvtHandler)
+			CSocketEventDispatcher::Get().RemovePending(m_pEvtHandler);
 	}
 	m_state = none;
 
@@ -1269,7 +1452,28 @@ int CSocket::Peek(void* buffer, unsigned int size, int& error)
 
 int CSocket::Write(const void* buffer, unsigned int size, int& error)
 {
-	int res = send(m_fd, (const char*)buffer, size, 0);
+#ifdef MSG_NOSIGNAL
+	const int flags = MSG_NOSIGNAL;
+#else
+	const int flags = 0;
+
+#if !defined(SO_NOSIGPIPE) && !defined(__WXMSW__)
+	// Some systems have neither. Need to block signal
+	struct sigaction old_action;
+	struct sigaction action = {0};
+	action.sa_handler = SIG_IGN;
+	int signal_set = sigaction(SIGPIPE, &action, &old_action);
+#endif
+
+#endif
+
+	int res = send(m_fd, (const char*)buffer, size, flags);
+
+#if !defined(MSG_NOSIGNAL) && !defined(SO_NOSIGPIPE) && !defined(__WXMSW__)
+	// Restore previous signal handler
+	if (!signal_set)
+		sigaction(SIGPIPE, &old_action, 0);
+#endif
 
 	if (res == -1)
 	{
@@ -1455,6 +1659,7 @@ int CSocket::Listen(int family, int port /*=0*/)
 #endif
 			m_fd = -1;
 		}
+		freeaddrinfo(addressList);
 		if (m_fd == -1)
 			return res;
 	}
@@ -1559,12 +1764,18 @@ CSocket* CSocket::Accept(int &error)
 #endif
 		return 0;
 	}
+	
+#if defined(SO_NOSIGPIPE) && !defined(MSG_NOSIGNAL)
+	// We do not want SIGPIPE if writing to socket.
+	const int value = 1;
+	setsockopt(fd, SOL_SOCKET, SO_NOSIGPIPE, &value, sizeof(int));
+#endif
 
 	SetNonblocking(fd);
 
 	DoSetBufferSizes(fd, m_buffer_sizes[0], m_buffer_sizes[1]);
 
-	CSocket* pSocket = new CSocket(0, -1);
+	CSocket* pSocket = new CSocket(0);
 	pSocket->m_state = connected;
 	pSocket->m_fd = fd;
 	pSocket->m_pSocketThread = new CSocketThread();
