@@ -6,11 +6,7 @@
 #include <gnutls/x509.h>
 #include <errno.h>
 
-BEGIN_EVENT_TABLE(CTlsSocket, wxEvtHandler)
-EVT_FZ_SOCKET(wxID_ANY, CTlsSocket::OnSocketEvent)
-END_EVENT_TABLE()
-
-CTlsSocket::CTlsSocket(wxEvtHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
+CTlsSocket::CTlsSocket(CSocketEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
 	: CBackend(pEvtHandler), m_pOwner(pOwner)
 {
 	m_pSocket = pSocket;
@@ -29,10 +25,6 @@ CTlsSocket::CTlsSocket(wxEvtHandler* pEvtHandler, CSocket* pSocket, CControlSock
 
 	m_tlsState = noconn;
 
-	m_lastError = 0;
-	m_lastCount = 0;
-	m_lastSuccessful = false;
-
 	m_lastReadFailed = true;
 	m_lastWriteFailed = false;
 	m_writeSkip = 0;
@@ -59,13 +51,14 @@ CTlsSocket::~CTlsSocket()
 bool CTlsSocket::Init()
 {
 	// This function initializes GnuTLS
+	m_initialized = true;
 	int res = gnutls_global_init();
 	if (res)
 	{
 		LogError(res);
+		Uninit();
 		return false;
 	}
-	m_initialized = true;
 
 	res = gnutls_certificate_allocate_credentials(&m_certCredentials);
 	if (res < 0)
@@ -209,11 +202,11 @@ ssize_t CTlsSocket::PushFunction(const void* data, size_t len)
 		return -1;
 	}
 
-	m_pSocketBackend->Write(data, len);
+	int error;
+	int written = m_pSocketBackend->Write(data, len, error);
 
-	if (m_pSocketBackend->Error())
+	if (written < 0)
 	{
-		const int error = m_pSocketBackend->LastError();
 		if (error == EAGAIN)
 		{
 			m_canWriteToSocket = false;
@@ -225,7 +218,7 @@ ssize_t CTlsSocket::PushFunction(const void* data, size_t len)
 		return -1;
 	}
 
-	return m_pSocketBackend->LastCount();
+	return written;
 }
 
 ssize_t CTlsSocket::PullFunction(void* data, size_t len)
@@ -245,30 +238,30 @@ ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 		return -1;
 	}
 
-	m_pSocketBackend->Read(data, len);
-	if (m_pSocketBackend->Error())
+	int error;
+	int read = m_pSocketBackend->Read(data, len, error);
+	if (read < 0)
 	{
-		if (m_pSocketBackend->LastError() == EAGAIN)
+		if (error == EAGAIN)
 		{
 			m_canReadFromSocket = false;
 			if (m_canCheckCloseSocket && !m_pSocketBackend->IsWaiting(CRateLimiter::inbound))
 			{
-				CSocketEvent evt(m_pSocketBackend->GetId(), CSocketEvent::close);
-				wxPostEvent(this, evt);
+				CSocketEvent *evt = new CSocketEvent(this, m_pSocketBackend, CSocketEvent::close);
+				CSocketEventDispatcher::Get().SendEvent(evt);
 			}
 		}
 
-		gnutls_transport_set_errno(m_session, m_pSocketBackend->LastError());
+		gnutls_transport_set_errno(m_session, error);
 		return -1;
 	}
 
 	if (m_canCheckCloseSocket)
 	{
-		CSocketEvent evt(m_pSocketBackend->GetId(), CSocketEvent::close);
-		wxPostEvent(this, evt);
+		CSocketEvent *evt = new CSocketEvent(this, m_pSocketBackend, CSocketEvent::close);
+		CSocketEventDispatcher::Get().SendEvent(evt);
 	}
 
-	unsigned int read = m_pSocketBackend->LastCount();
 	if (!read)
 		m_socket_eof = true;
 
@@ -281,9 +274,6 @@ void CTlsSocket::OnSocketEvent(CSocketEvent& event)
 	if (!m_session)
 		return;
 
-	if (event.GetId() != m_pSocketBackend->GetId())
-		return;
-
 	switch (event.GetType())
 	{
 	case CSocketEvent::read:
@@ -296,12 +286,11 @@ void CTlsSocket::OnSocketEvent(CSocketEvent& event)
 		{
 			m_canCheckCloseSocket = true;
 			char tmp[100];
-			m_pSocketBackend->Peek(&tmp, 100);
-			if (!m_pSocketBackend->Error())
+			int error;
+			int peeked = m_pSocketBackend->Peek(&tmp, 100, error);
+			if (peeked >= 0)
 			{
-				int lastCount = m_pSocketBackend->LastCount();
-
-				if (lastCount)
+				if (peeked > 0)
 					m_pOwner->LogMessage(Debug_Verbose, _T("CTlsSocket::OnSocketEvent(): pending data, postponing close event"));
 				else
 				{
@@ -310,15 +299,15 @@ void CTlsSocket::OnSocketEvent(CSocketEvent& event)
 				}
 				OnRead();
 
-				if (lastCount)
+				if (peeked)
 					return;
 			}
 
 			m_pOwner->LogMessage(Debug_Info, _T("CTlsSocket::OnSocketEvent(): close event received"));
 
 			//Uninit();
-			CSocketEvent evt(GetId(), CSocketEvent::close);
-			wxPostEvent(m_pEvtHandler, evt);
+			CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close);
+			CSocketEventDispatcher::Get().SendEvent(evt);
 		}
 		break;
 	default:
@@ -455,11 +444,11 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/, bool try_resu
 
 		if (m_shutdown_requested)
 		{
-			Shutdown();
-			if (!Error() || LastError() != EAGAIN)
+			int error = Shutdown();
+			if (!error || error != EAGAIN)
 			{
-				CSocketEvent evt(GetId(), CSocketEvent::close);
-				wxPostEvent(m_pEvtHandler, evt);
+				CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close);
+				CSocketEventDispatcher::Get().SendEvent(evt);
 			}
 		}
 
@@ -473,26 +462,22 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/, bool try_resu
 	return FZ_REPLY_ERROR;
 }
 
-void CTlsSocket::Read(void *buffer, unsigned int len)
+int CTlsSocket::Read(void *buffer, unsigned int len, int& error)
 {
 	if (m_tlsState == handshake || m_tlsState == verifycert)
 	{
-		m_lastError = EAGAIN;
-		m_lastSuccessful = false;
-		m_lastReadFailed = true;
-		return;
+		error = EAGAIN;
+		return -1;
 	}
 	else if (m_tlsState != conn)
 	{
-		m_lastError = ENOTCONN;
-		m_lastSuccessful = false;
-		return;
+		error = ENOTCONN;
+		return -1;
 	}
 	else if (m_lastReadFailed)
 	{
-		m_lastError = EAGAIN;
-		m_lastSuccessful = false;
-		return;
+		error = EAGAIN;
+		return -1;
 	}
 
 	if (m_peekDataLen)
@@ -514,17 +499,13 @@ void CTlsSocket::Read(void *buffer, unsigned int len)
 
 		TriggerEvents();
 
-		m_lastSuccessful = true;
-		m_lastCount = min;
-		return;
+		error = 0;
+		return min;
 	}
 
 	int res = gnutls_record_recv(m_session, buffer, len);
 	if (res >= 0)
 	{
-		m_lastSuccessful = true;
-		m_lastCount = res;
-
 		if (res > 0)
 			TriggerEvents();
 		else
@@ -537,52 +518,47 @@ void CTlsSocket::Read(void *buffer, unsigned int len)
 			// notification.
 		}
 
-		return;
+		error = 0;
+		return res;
 	}
 
 	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN)
 	{
-		m_lastError = EAGAIN;
+		error = EAGAIN;
 		m_lastReadFailed = true;
 	}
 	else
 	{
 		Failure(res, 0);
-		m_lastError = ECONNABORTED;
+		error = ECONNABORTED;
 	}
 
-	m_lastSuccessful = false;
-	return;
+	return -1;
 }
 
-void CTlsSocket::Write(const void *buffer, unsigned int len)
+int CTlsSocket::Write(const void *buffer, unsigned int len, int& error)
 {
 	if (m_tlsState == handshake || m_tlsState == verifycert)
 	{
-		m_lastError = EAGAIN;
-		m_lastSuccessful = false;
-		return;
+		error = EAGAIN;
+		return -1;
 	}
 	else if (m_tlsState != conn)
 	{
-		m_lastError = ENOTCONN;
-		m_lastSuccessful = false;
-		return;
+		error = ENOTCONN;
+		return -1;
 	}
 
 	if (m_lastWriteFailed)
 	{
-		m_lastError = EAGAIN;
-		m_lastSuccessful = false;
-		return;
+		error = EAGAIN;
+		return -1;
 	}
 
 	if (m_writeSkip >= len)
 	{
 		m_writeSkip -= len;
-		m_lastCount = len;
-		m_lastSuccessful = true;
-		return;
+		return len;
 	}
 
 	len -= m_writeSkip;
@@ -591,34 +567,35 @@ void CTlsSocket::Write(const void *buffer, unsigned int len)
 	int res = gnutls_record_send(m_session, buffer, len);
 	if (res >= 0)
 	{
-		m_lastSuccessful = true;
-		m_lastCount = res + m_writeSkip;
+		error = 0;
+		int written = res + m_writeSkip;
 		m_writeSkip = 0;
 
 		TriggerEvents();
-		return;
+		return written;
 	}
 
 	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN)
 	{
 		if (m_writeSkip)
 		{
-			m_lastSuccessful = true;
-			m_lastCount = m_writeSkip;
+			error = 0;
+			int written = m_writeSkip;
 			m_writeSkip = 0;
+			return written;
 		}
 		else
 		{
-			m_lastSuccessful = false;
-			m_lastError = EAGAIN;
+			error = EAGAIN;
 			m_lastWriteFailed = true;
+			return -1;
 		}
 	}
 	else
 	{
-		m_lastSuccessful = false;
 		Failure(res, 0);
-		m_lastError = ECONNABORTED;
+		error = ECONNABORTED;
+		return -1;
 	}
 }
 
@@ -629,15 +606,15 @@ void CTlsSocket::TriggerEvents()
 
 	if (m_canTriggerRead)
 	{
-		CSocketEvent evt(GetId(), CSocketEvent::read);
-		wxPostEvent(m_pEvtHandler, evt);
+		CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::read);
+		CSocketEventDispatcher::Get().SendEvent(evt);
 		m_canTriggerRead = false;
 	}
 
 	if (m_canTriggerWrite)
 	{
-		CSocketEvent evt(GetId(), CSocketEvent::write);
-		wxPostEvent(m_pEvtHandler, evt);
+		CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::write);
+		CSocketEventDispatcher::Get().SendEvent(evt);
 		m_canTriggerWrite = false;
 	}
 }
@@ -706,65 +683,53 @@ void CTlsSocket::Failure(int code, int socket_error)
 
 	if (socket_error)
 	{
-		CSocketEvent evt(GetId(), CSocketEvent::close, socket_error);
-		wxPostEvent(m_pEvtHandler, evt);
+		CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, socket_error);
+		CSocketEventDispatcher::Get().SendEvent(evt);
 	}
 }
 
-void CTlsSocket::Peek(void *buffer, unsigned int len)
+int CTlsSocket::Peek(void *buffer, unsigned int len, int& error)
 {
 	if (m_peekData)
 	{
 		int min = wxMin(len, m_peekDataLen);
 		memcpy(buffer, m_peekData, min);
 
-		m_lastCount = min;
-		m_lastSuccessful = true;
-		return;
+		error = 0;
+		return min;
 	}
 
-	Read(buffer, len);
-	if (Error() || !LastCount())
-		return;
+	int read = Read(buffer, len, error);
+	if (read <= 0)
+		return read;
 
-	m_peekDataLen = LastCount();
+	m_peekDataLen = read;
 	m_peekData = new char[m_peekDataLen];
 	memcpy(m_peekData, buffer, m_peekDataLen);
+
+	return read;
 }
 
-void CTlsSocket::Shutdown()
+int CTlsSocket::Shutdown()
 {
 	m_pOwner->LogMessage(Debug_Verbose, _T("CTlsSocket::Shutdown()"));
 
 	if (m_tlsState == closed)
-	{
-		m_lastSuccessful = true;
-		return;
-	}
+		return 0;
 
 	if (m_tlsState == closing)
-	{
-		m_lastSuccessful = false;
-		m_lastError = EAGAIN;
-		return;
-	}
+		return EAGAIN;
 
 	if (m_tlsState == handshake || m_tlsState == verifycert)
 	{
 		// Shutdown during handshake is not a good idea.
 		m_pOwner->LogMessage(Debug_Verbose, _T("Shutdown during handshake, postponing"));
 		m_shutdown_requested = true;
-		m_lastSuccessful = false;
-		m_lastError = EAGAIN;
-		return;
+		return EAGAIN;
 	}
 
 	if (m_tlsState != conn)
-	{
-		m_lastSuccessful = false;
-		m_lastError = ECONNABORTED;
-		return;
-	}
+		return ECONNABORTED;
 
 	m_tlsState = closing;
 
@@ -772,18 +737,14 @@ void CTlsSocket::Shutdown()
 	if (!res)
 	{
 		m_tlsState = closed;
-		m_lastSuccessful = true;
-		return;
+		return 0;
 	}
 
-	m_lastSuccessful = false;
 	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN)
-		m_lastError = EAGAIN;
-	else
-	{
-		Failure(res, 0);
-		m_lastError = ECONNABORTED;
-	}
+		return EAGAIN;
+
+	Failure(res, 0);
+	return ECONNABORTED;
 }
 
 void CTlsSocket::ContinueShutdown()
@@ -795,8 +756,8 @@ void CTlsSocket::ContinueShutdown()
 	{
 		m_tlsState = closed;
 
-		CSocketEvent evt(GetId(), CSocketEvent::close);
-		wxPostEvent(m_pEvtHandler, evt);
+		CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close);
+		CSocketEventDispatcher::Get().SendEvent(evt);
 
 		return;
 	}
@@ -823,8 +784,8 @@ void CTlsSocket::TrustCurrentCert(bool trusted)
 
 		if (m_tlsState == conn)
 		{
-			CSocketEvent evt(GetId(), CSocketEvent::connection);
-			wxPostEvent(m_pEvtHandler, evt);
+			CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection);
+			CSocketEventDispatcher::Get().SendEvent(evt);
 		}
 
 		TriggerEvents();
