@@ -19,7 +19,6 @@
 #include "servercapabilities.h"
 
 BEGIN_EVENT_TABLE(CTransferSocket, wxEvtHandler)
-	EVT_FZ_SOCKET(wxID_ANY, CTransferSocket::OnSocketEvent)
 	EVT_IOTHREAD(wxID_ANY, CTransferSocket::OnIOThreadEvent)
 END_EVENT_TABLE();
 
@@ -130,7 +129,7 @@ wxString CTransferSocket::SetupActiveTransfer(const wxString& ip)
 
 void CTransferSocket::OnSocketEvent(CSocketEvent &event)
 {
-	if (m_pProxyBackend && event.GetId() == m_pProxyBackend->GetId())
+	if (m_pProxyBackend)
 	{
 		switch (event.GetType())
 		{
@@ -164,7 +163,7 @@ void CTransferSocket::OnSocketEvent(CSocketEvent &event)
 		return;
 	}
 
-	if (m_pSocketServer && event.GetId() == m_pSocketServer->GetId())
+	if (m_pSocketServer)
 	{
 		if (event.GetType() == CSocketEvent::connection)
 			OnAccept(event.GetError());
@@ -173,23 +172,6 @@ void CTransferSocket::OnSocketEvent(CSocketEvent &event)
 		return;
 	}
 
-	if (m_pBackend)
-	{
-		if (event.GetId() != m_pBackend->GetId())
-		{
-			m_pControlSocket->LogMessage(::Debug_Info, _T("Skipping socket event %d, id mismatch."), event.GetType());
-			return;
-		}
-	}
-	else
-	{
-		if (!m_pSocket || m_pSocket->GetId() != event.GetId())
-		{
-			m_pControlSocket->LogMessage(m_pSocket ? ::Debug_Debug : ::Debug_Info, _T("Skipping socket event %d, no socket or id mismatch."), event.GetType());
-			return;
-		}
-	}
-
 	switch (event.GetType())
 	{
 	case CSocketEvent::connection:
@@ -296,24 +278,21 @@ void CTransferSocket::OnReceive()
 		while (true)
 		{
 			char *pBuffer = new char[4096];
-			m_pBackend->Read(pBuffer, 4096);
-			if (m_pBackend->Error())
+			int error;
+			int numread = m_pBackend->Read(pBuffer, 4096, error);
+			if (numread < 0)
 			{
 				delete [] pBuffer;
-				int error = m_pBackend->LastError();
 				if (error != EAGAIN)
 				{
 					m_pControlSocket->LogMessage(::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error).c_str());
 					TransferEnd(transfer_failure);
 				}
 				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
-				{
-					CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
-					AddPendingEvent(evt);
-				}
+					TransferEnd(successful);
+
 				return;
 			}
-			int numread = m_pBackend->LastCount();
 
 			if (numread > 0)
 			{
@@ -325,22 +304,11 @@ void CTransferSocket::OnReceive()
 					m_pControlSocket->SetTransferStatusMadeProgress();
 				}
 				m_pControlSocket->UpdateTransferStatus(numread);
-				if (m_onCloseCalled)
-				{
-					CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
-					AddPendingEvent(evt);
-				}
 			}
 			else
 			{
 				delete [] pBuffer;
-				if (!numread)
-					TransferEnd(successful);
-				else if (numread < 0)
-				{
-					m_pControlSocket->LogMessage(Debug_Warning, _T("  numread < 0"));
-					TransferEnd(transfer_failure);
-				}
+				TransferEnd(successful);
 				return;
 			}
 		}
@@ -352,23 +320,19 @@ void CTransferSocket::OnReceive()
 			if (!CheckGetNextWriteBuffer())
 				return;
 
-			m_pBackend->Read(m_pTransferBuffer, m_transferBufferLen);
-			if (m_pBackend->Error())
+			int error;
+			int numread = m_pBackend->Read(m_pTransferBuffer, m_transferBufferLen, error);
+			if (numread < 0)
 			{
-				int error = m_pBackend->LastError();
 				if (error != EAGAIN)
 				{
 					m_pControlSocket->LogMessage(::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error).c_str());
 					TransferEnd(transfer_failure);
 				}
 				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
-				{
-					CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
-					AddPendingEvent(evt);
-				}
+					TransferEnd(successful);
 				return;
 			}
-			int numread = m_pBackend->LastCount();
 
 			if (numread > 0)
 			{
@@ -385,12 +349,6 @@ void CTransferSocket::OnReceive()
 
 				if (!CheckGetNextWriteBuffer())
 					return;
-
-				if (m_onCloseCalled && m_transferEndReason == none)
-				{
-					CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
-					AddPendingEvent(evt);
-				}
 			}
 			else //!numread
 			{
@@ -404,10 +362,10 @@ void CTransferSocket::OnReceive()
 		while (true)
 		{
 			char buffer[2];
-			m_pBackend->Read(buffer, 2);
-			if (m_pBackend->Error())
+			int error;
+			int numread = m_pBackend->Read(buffer, 2, error);
+			if (numread < 0)
 			{
-				const int error = m_pBackend->LastError();
 				if (error != EAGAIN)
 				{
 					m_pControlSocket->LogMessage(::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error).c_str());
@@ -415,33 +373,36 @@ void CTransferSocket::OnReceive()
 				}
 				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
 				{
-					CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
-					AddPendingEvent(evt);
+					if (m_transferBufferLen == 1)
+						TransferEnd(successful);
+					else
+					{
+						m_pControlSocket->LogMessage(::Debug_Warning, _T("Server incorrectly sent %d bytes"), m_transferBufferLen);
+						TransferEnd(failed_resumetest);
+					}
 				}
 				return;
 			}
-			int numread = m_pBackend->LastCount();
+
 			if (!numread)
 			{
 				if (m_transferBufferLen == 1)
 					TransferEnd(successful);
 				else
+				{
+					m_pControlSocket->LogMessage(::Debug_Warning, _T("Server incorrectly sent %d bytes"), m_transferBufferLen);
 					TransferEnd(failed_resumetest);
+				}
 				return;
 			}
 			m_transferBufferLen += numread;
 
 			if (m_transferBufferLen > 1)
 			{
+				m_pControlSocket->LogMessage(::Debug_Warning, _T("Server incorrectly sent %d bytes"), m_transferBufferLen);
 				TransferEnd(failed_resumetest);
 				return;
 			}
-			else if (m_onCloseCalled)
-			{
-				CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
-				AddPendingEvent(evt);
-				return;
-			}
 		}
 	}
 }
@@ -461,37 +422,33 @@ void CTransferSocket::OnSend()
 	if (m_transferMode != upload)
 		return;
 
-	int numsent = 0;
+	int error;
+	int written;
 	do
 	{
 		if (!CheckGetNextReadBuffer())
 			return;
 
-		m_pBackend->Write(m_pTransferBuffer, m_transferBufferLen);
-		if (m_pBackend->Error())
+		written = m_pBackend->Write(m_pTransferBuffer, m_transferBufferLen, error);
+		if (written <= 0)
 			break;
 
-		numsent = m_pBackend->LastCount();
-		if (numsent > 0)
+		m_pEngine->SetActive(false);
+		if (m_madeProgress == 1)
 		{
-			m_pEngine->SetActive(false);
-			if (m_madeProgress == 1)
-			{
-				m_pControlSocket->LogMessage(::Debug_Debug, _T("Made progress in CTransferSocket::OnSend()"));
-				m_madeProgress = 2;
-				m_pControlSocket->SetTransferStatusMadeProgress();
-			}
-			m_pControlSocket->UpdateTransferStatus(numsent);
+			m_pControlSocket->LogMessage(::Debug_Debug, _T("Made progress in CTransferSocket::OnSend()"));
+			m_madeProgress = 2;
+			m_pControlSocket->SetTransferStatusMadeProgress();
 		}
+		m_pControlSocket->UpdateTransferStatus(written);
 
-		m_pTransferBuffer += numsent;
-		m_transferBufferLen -= numsent;
+		m_pTransferBuffer += written;
+		m_transferBufferLen -= written;
 	}
-	while (!m_pBackend->Error() && numsent > 0);
+	while (true);
 
-	if (m_pBackend->Error())
+	if (written < 0)
 	{
-		int error = m_pBackend->LastError();
 		if (error == EAGAIN)
 		{
 			if (!m_madeProgress)
@@ -530,8 +487,7 @@ void CTransferSocket::OnClose(int error)
 	{
 		if (m_shutdown && m_pTlsSocket)
 		{
-			m_pTlsSocket->Shutdown();
-			if (m_pTlsSocket->Error())
+			if (m_pTlsSocket->Shutdown() != 0)
 				TransferEnd(transfer_failure);
 			else
 				TransferEnd(successful);
@@ -549,8 +505,8 @@ void CTransferSocket::OnClose(int error)
 	}
 
 	char buffer[100];
-	m_pBackend->Peek(&buffer, 100);
-	if (!m_pBackend->Error() && m_pBackend->LastCount())
+	int numread = m_pBackend->Peek(&buffer, 100, error);
+	if (numread > 0)
 	{
 #ifndef __WXMSW__
 		wxFAIL_MSG(_T("Peek isn't supposed to return data after close notification"));
@@ -565,7 +521,7 @@ void CTransferSocket::OnClose(int error)
 
 		return;
 	}
-	else if (m_pBackend->Error() && m_pBackend->LastError() != EAGAIN)
+	else if (numread < 0 && error != EAGAIN)
 	{
 		m_pControlSocket->LogMessage(::Error, _("Transfer connection interrupted: %s"), CSocket::GetErrorDescription(error).c_str());
 		TransferEnd(transfer_failure);
@@ -591,7 +547,7 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 	delete m_pSocketServer;
 	m_pSocketServer = 0;
 
-	m_pSocket = new CSocket(this, 0);
+	m_pSocket = new CSocket(this);
 
 	if (m_pControlSocket->m_pProxyBackend)
 	{
@@ -610,8 +566,8 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 			return false;
 		}
 		int error;
-		host = m_pControlSocket->GetPeerIP();
-		port = m_pControlSocket->GetRemotePort(error);
+		host = m_pControlSocket->m_pSocket->GetPeerIP();
+		port = m_pControlSocket->m_pSocket->GetRemotePort(error);
 	}
 
 	SetSocketBufferSizes(m_pSocket);
@@ -631,6 +587,8 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 
 void CTransferSocket::SetActive()
 {
+	if (m_transferEndReason != none)
+		return;
 	if (m_transferMode == download || m_transferMode == upload)
 	{
 		CFtpFileTransferOpData *pData = static_cast<CFtpFileTransferOpData *>(static_cast<CRawTransferOpData *>(m_pControlSocket->m_pCurOpData)->pOldData);;
@@ -639,7 +597,10 @@ void CTransferSocket::SetActive()
 	}
 
 	m_bActive = true;
-	if (m_pSocket && m_pSocket->GetState() == CSocket::connected)
+	if (!m_pSocket)
+		return;
+	
+	if (m_pSocket->GetState() == CSocket::connected || m_pSocket->GetState() == CSocket::closing)
 		TriggerPostponedEvents();
 }
 
@@ -679,8 +640,8 @@ void CTransferSocket::TransferEnd(enum TransferEndReason reason)
 
 CSocket* CTransferSocket::CreateSocketServer(int port)
 {
-	CSocket* pServer = new CSocket(this, CBackend::GetNextId());
-	int res = pServer->Listen(m_pControlSocket->GetAddressFamily(), port);
+	CSocket* pServer = new CSocket(this);
+	int res = pServer->Listen(m_pControlSocket->m_pSocket->GetAddressFamily(), port);
 	if (res)
 	{
 		m_pControlSocket->LogMessage(::Debug_Verbose, _T("Could not listen on port %d: %s"), port, CSocket::GetErrorDescription(res).c_str());
@@ -785,10 +746,11 @@ bool CTransferSocket::CheckGetNextReadBuffer()
 			if (m_pTlsSocket)
 			{
 				m_shutdown = true;
-				m_pTlsSocket->Shutdown();
-				if (m_pTlsSocket->Error())
+				
+				int error = m_pTlsSocket->Shutdown();
+				if (error != 0)
 				{
-					if (m_pTlsSocket->LastError() != EAGAIN)
+					if (error != EAGAIN)
 						TransferEnd(transfer_failure);
 					return false;
 				}
@@ -869,13 +831,19 @@ void CTransferSocket::TriggerPostponedEvents()
 		m_pControlSocket->LogMessage(::Debug_Verbose, _T("Executing postponed receive"));
 		m_postponedReceive = false;
 		OnReceive();
+		if (m_transferEndReason != none)
+			return;
 	}
 	if (m_postponedSend)
 	{
 		m_pControlSocket->LogMessage(::Debug_Verbose, _T("Executing postponed send"));
 		m_postponedSend = false;
 		OnSend();
+		if (m_transferEndReason != none)
+			return;
 	}
+	if (m_onCloseCalled)
+		OnClose(0);
 }
 
 bool CTransferSocket::InitBackend()
