@@ -45,6 +45,7 @@ enum Command
 #define FZ_REPLY_TIMEOUT		(0x0800 | FZ_REPLY_ERROR)
 #define FZ_REPLY_NOTSUPPORTED	(0x1000 | FZ_REPLY_ERROR) // Will be returned if command not supported by that protocol
 #define FZ_REPLY_WRITEFAILED	(0x2000 | FZ_REPLY_ERROR) // Happens if local file could not be written during transfer
+#define FZ_REPLY_LINKNOTDIR		(0x4000)
 
 // Small macro to simplify command class declaration
 // Basically all this macro does, is to declare the class and add the required
@@ -85,24 +86,40 @@ DECLARE_COMMAND(CDisconnectCommand, cmd_disconnect)
 DECLARE_COMMAND(CCancelCommand, cmd_cancel)
 };
 
+#define LIST_FLAG_REFRESH 1
+#define LIST_FLAG_AVOID 2
+#define LIST_FLAG_FALLBACK_CURRENT 4
+#define LIST_FLAG_LINK 8
 DECLARE_COMMAND(CListCommand, cmd_list)
     // Without a given directory, the current directory will be listed.
     // Directories can either be given as absolute path or as
     // pair of an absolute path and the very last path segments.
-	// Set refresh to true to get a directory listing even if a cache
+	
+	// Set LIST_FLAG_REFRESH to get a directory listing even if a cache
 	// lookup can be made after finding out true remote directory.
-	CListCommand(bool refresh = false);
-	CListCommand(CServerPath path, wxString subDir = _T(""), bool refresh = false);
+	//
+	// Set LIST_FLAG_AVOID to get a directory listing only if cache lookup
+	// fails or contains unsure entries, otherwise don't send listing.
+	//
+	// If LIST_FLAG_FALLBACK_CURRENT is set and CWD fails, list whatever
+	// directory we are currently in. Useful for initial reconnect to the
+	// server when we don't know if remote directory still exists
+	//
+	// LIST_FLAG_LINK is used for symlink discovery. There's unfortunately
+	// no sane way to distinguish between symlinks to files and symlinks to
+	// directories.
+	CListCommand(int flags = 0);
+	CListCommand(CServerPath path, wxString subDir = _T(""), int flags = 0);
 	
 	CServerPath GetPath() const;
 	wxString GetSubDir() const;
 
-	bool Refresh() const;
+	int GetFlags() const { return m_flags; }
 
 protected:
 	CServerPath m_path;
 	wxString m_subDir;
-	bool m_refresh;
+	int m_flags;
 };
 
 DECLARE_COMMAND(CFileTransferCommand, cmd_transfer)
