@@ -44,6 +44,8 @@ public:
 	// directory.
 	void InvalidateCurrentWorkingDirs(const CServerPath& path);
 
+	int GetEngineId() const {return m_engine_id; }
+
 protected:
 	CFileZillaEnginePrivate();
 	virtual ~CFileZillaEnginePrivate();
@@ -63,12 +65,13 @@ protected:
 
 	int ContinueConnect();
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnEngineEvent(wxFzEngineEvent &event);
 	void OnTimer(wxTimerEvent& event);
 
 	wxEvtHandler *m_pEventHandler;
 
+	int m_engine_id;
 	static std::list<CFileZillaEnginePrivate*> m_engineList;
 
 	// Indicicates if data has been received/sent and whether to send any notifications
