@@ -7,6 +7,7 @@
 #include "dndobjects.h"
 #include "inputdialog.h"
 #include "local_filesys.h"
+#include "dragdropmanager.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -15,8 +16,16 @@
 #ifdef __WXMSW__
 #include <wx/msw/registry.h>
 #include <shlobj.h>
+#include <dbt.h>
 #endif
 
+class CTreeItemData : public wxTreeItemData
+{
+public:
+	CTreeItemData(const wxString& known_subdir) : m_known_subdir(known_subdir) {}
+	wxString m_known_subdir;
+};
+
 class CLocalTreeViewDropTarget : public wxDropTarget
 {
 public:
@@ -74,15 +83,19 @@ public:
 		if (!hit)
 			return wxDragNone;
 
-		wxString dir = GetDirFromItem(hit);
-		if (dir == _T("") || !CState::LocalDirIsWriteable(dir))
+		const CLocalPath path(GetDirFromItem(hit));
+		if (path.empty() || !path.IsWriteable())
 			return wxDragNone;
 
 		if (!GetData())
 			return wxDragError;
 
+		CDragDropManager* pDragDropManager = CDragDropManager::Get();
+		if (pDragDropManager)
+			pDragDropManager->pDropTarget = m_pLocalTreeView;
+
 		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
-			m_pLocalTreeView->m_pState->HandleDroppedFiles(m_pFileDataObject, dir, def == wxDragCopy);
+			m_pLocalTreeView->m_pState->HandleDroppedFiles(m_pFileDataObject, path, def == wxDragCopy);
 		else
 		{
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
@@ -97,7 +110,7 @@ public:
 				return wxDragNone;
 			}
 
-			if (!m_pLocalTreeView->m_pState->DownloadDroppedFiles(m_pRemoteDataObject, dir))
+			if (!m_pLocalTreeView->m_pState->DownloadDroppedFiles(m_pRemoteDataObject, path))
 				return wxDragNone;
 		}
 
@@ -112,8 +125,8 @@ public:
 		if (!hit)
 			return false;
 
-		wxString dir = GetDirFromItem(hit);
-		if (dir == _T("") || !CState::LocalDirIsWriteable(dir))
+		const wxString dir = GetDirFromItem(hit);
+		if (dir == _T("") || !CLocalPath(dir).IsWriteable())
 			return false;
 
 		return true;
@@ -212,6 +225,10 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState,
 	CStateEventHandler(pState),
 	m_pQueueView(pQueueView)
 {
+#ifdef __WXMAC__
+	SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
+#endif
+
 	pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
 	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
 
@@ -323,8 +340,11 @@ wxTreeItemId CLocalTreeView::GetNearestParent(wxString& localDir)
 		root = GetNextSibling(root);
 	}
 	if (!root)
-		// TODO: Number is drives changed
+	{
+		if (drive[1] == ':')
+			return AddDrive(drive[0]);
 		return wxTreeItemId();
+	}
 #else
 		if (localDir[0] == '/')
 			localDir = localDir.Mid(1);
@@ -410,14 +430,14 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 		// Check if drive should be hidden by default
 		if (pDrive[0] != 0 && pDrive[1] == ':')
 		{
-			int bit = -1;
+			int bit = 0;
 			char letter = pDrive[0];
 			if (letter >= 'A' && letter <= 'Z')
 				bit = 1 << (letter - 'A');
 			if (letter >= 'a' && letter <= 'z')
 				bit = 1 << (letter - 'a');
 
-			if (bit != -1 && drivesToHide & bit)
+			if (drivesToHide & bit)
 			{
 				pDrive += wxStrlen(pDrive) + 1;
 				continue;
@@ -458,8 +478,7 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 
 				const wxString fullName = dirname + knownSubdir;
 				item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName), GetIconIndex(opened_dir, fullName));
-				if (HasSubdir(fullName))
-					AppendItem(item, _T(""));
+				CheckSubdirStatus(item, fullName);
 			}
 			else
 			{
@@ -508,22 +527,20 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 			matchedKnown = true;
 
 		wxTreeItemId item = AppendItem(parent, file, GetIconIndex(::dir, fullName), GetIconIndex(opened_dir, fullName));
-		if (HasSubdir(fullName))
-			AppendItem(item, _T(""));
+		CheckSubdirStatus(item, fullName);
 	}
 
 	if (!matchedKnown && knownSubdir != _T(""))
 	{
 		const wxString fullName = dirname + knownSubdir;
 		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName), GetIconIndex(opened_dir, fullName));
-		if (HasSubdir(fullName))
-			AppendItem(item, _T(""));
+		CheckSubdirStatus(item, fullName);
 	}
 
 	SortChildren(parent);
 }
 
-bool CLocalTreeView::HasSubdir(const wxString& dirname)
+wxString CLocalTreeView::HasSubdir(const wxString& dirname)
 {
 	wxLogNull nullLog;
 
@@ -531,7 +548,7 @@ bool CLocalTreeView::HasSubdir(const wxString& dirname)
 	
 	CLocalFileSystem local_filesystem;
 	if (!local_filesystem.BeginFindFiles(dirname, true))
-		return false;
+		return _T("");
 
 	wxString file;
 	bool wasLink;
@@ -547,14 +564,13 @@ bool CLocalTreeView::HasSubdir(const wxString& dirname)
 			continue;
 		}
 
-		wxString fullName = dirname + file;
 		if (filter.FilenameFiltered(file, dirname, true, size, true, attributes))
 			continue;
 
-		return true;
+		return file;
 	}
 
-	return false;
+	return _T("");
 }
 
 wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, wxString dirname, wxString subDir)
@@ -776,21 +792,11 @@ void CLocalTreeView::Refresh()
 			{
 				if (!IsExpanded(child))
 				{
-					wxTreeItemId subchild = GetLastChild(child);
-					if (subchild && GetItemText(subchild) == _T(""))
-					{
-						if (!HasSubdir(dir.dir + *iter + separator))
-							Delete(subchild);
-					}
-					else if (!subchild)
-					{
-						if (HasSubdir(dir.dir + *iter + separator))
-							AppendItem(child, _T(""));
-					}
-					else
+					wxString path = dir.dir + *iter + separator;
+					if (!CheckSubdirStatus(child, path))
 					{
 						t_dir subdir;
-						subdir.dir = dir.dir + *iter + separator;
+						subdir.dir = path;
 						subdir.item = child;
 						dirsToCheck.push_front(subdir);
 					}
@@ -819,8 +825,7 @@ void CLocalTreeView::Refresh()
 			{
 				wxString fullname = dir.dir + *iter + separator;
 				wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname), GetIconIndex(opened_dir, fullname));
-				if (HasSubdir(fullname))
-					AppendItem(newItem, _T(""));
+				CheckSubdirStatus(newItem, fullname);
 				iter++;
 				inserted = true;
 			}
@@ -839,8 +844,7 @@ void CLocalTreeView::Refresh()
 		{
 			wxString fullname = dir.dir + *iter + separator;
 			wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname), GetIconIndex(opened_dir, fullname));
-			if (HasSubdir(fullname))
-				AppendItem(newItem, _T(""));
+			CheckSubdirStatus(newItem, fullname);
 			iter++;
 			inserted = true;
 		}
@@ -894,7 +898,7 @@ int CLocalTreeView::OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId
 void CLocalTreeView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
 {
 	if (notification == STATECHANGE_LOCAL_DIR)
-		SetDir(m_pState->GetLocalDir());
+		SetDir(m_pState->GetLocalDir().GetPath());
 	else
 	{
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
@@ -929,6 +933,9 @@ void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 		return;
 #endif
 
+	CDragDropManager* pDragDropManager = CDragDropManager::Init();
+	pDragDropManager->pDragSource = this;
+
 	wxFileDataObject obj;
 
 	obj.AddFile(dir);
@@ -936,8 +943,17 @@ void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 	wxDropSource source(this);
 	source.SetData(obj);
 	int res = source.DoDragDrop(wxDrag_AllowMove);
-	if (res == wxDragCopy || res == wxDragMove)
+
+	bool handled_internally = pDragDropManager->pDropTarget != 0;
+
+	pDragDropManager->Release();
+
+	if (!handled_internally && (res == wxDragCopy || res == wxDragMove))
+	{
+		// We only need to refresh local side if the operation got handled
+		// externally, the internal handlers do this for us already
 		m_pState->RefreshLocal();
+	}
 }
 
 #ifndef __WXMSW__
@@ -1142,14 +1158,14 @@ bool CVolumeDescriptionEnumeratorThread::GetDrives()
 		// Check if drive should be hidden by default
 		if (pDrive[0] != 0 && pDrive[1] == ':')
 		{
-			int bit = -1;
+			int bit = 0;
 			char letter = pDrive[0];
 			if (letter >= 'A' && letter <= 'Z')
 				bit = 1 << (letter - 'A');
 			if (letter >= 'a' && letter <= 'z')
 				bit = 1 << (letter - 'a');
 
-			if (bit != -1 && drivesToHide & bit)
+			if (drivesToHide & bit)
 			{
 				pDrive += wxStrlen(pDrive) + 1;
 				continue;
@@ -1227,14 +1243,11 @@ void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 	if (!pMenu)
 		return;
 
-	const wxString& path = GetDirFromItem(m_contextMenuItem);
+	const CLocalPath path(GetDirFromItem(m_contextMenuItem));
+
+	const bool hasParent = path.HasParent();
+	const bool writeable = path.IsWriteable();
 
-	bool hasParent = m_pState->LocalDirHasParent(path);
-	const bool writeable = m_pState->LocalDirIsWriteable(path);
-#ifdef __WXMSW__
-	if (path.Len() == 3 && path.Mid(1) == _T(":\\"))
-		hasParent = false;
-#endif
 	const bool remoteConnected = m_pState->IsRemoteConnected() && !m_pState->GetRemotePath().IsEmpty();
 
 	pMenu->Enable(XRCID("ID_UPLOAD"), hasParent && remoteConnected);
@@ -1254,14 +1267,9 @@ void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 
 	wxString path = GetDirFromItem(m_contextMenuItem);
 
-	if (!m_pState->LocalDirHasParent(path))
+	if (!CLocalPath(path).HasParent())
 		return;
 
-#ifdef __WXMSW__
-	if (path.Len() == 3 && path.Mid(1) == _T(":\\"))
-		return;
-#endif
-
 	if (!m_pState->IsRemoteConnected())
 		return;
 
@@ -1287,7 +1295,7 @@ void CLocalTreeView::OnMenuMkdir(wxCommandEvent& event)
 	if (path.Last() != wxFileName::GetPathSeparator())
 		path += wxFileName::GetPathSeparator();
 
-	if (!m_pState->LocalDirIsWriteable(path))
+	if (!CLocalPath(path).IsWriteable())
 	{
 		wxBell();
 		return;
@@ -1337,16 +1345,8 @@ void CLocalTreeView::OnMenuRename(wxCommandEvent& event)
 	}
 #endif
 
-	wxString path = GetDirFromItem(m_contextMenuItem);
-
-#ifdef __WXMSW__
-	if (path.Len() == 3 && path.Mid(1) == _T(":\\"))
-	{
-		wxBell();
-		return;
-	}
-#endif
-	if (!m_pState->LocalDirHasParent(path) || !m_pState->LocalDirIsWriteable(path))
+	CLocalPath path(GetDirFromItem(m_contextMenuItem));
+	if (!path.HasParent() || !path.IsWriteable())
 	{
 		wxBell();
 		return;
@@ -1362,14 +1362,10 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent& event)
 
 	wxString path = GetDirFromItem(m_contextMenuItem);
 
-	if (!m_pState->LocalDirHasParent(path) || !m_pState->LocalDirIsWriteable(path))
+	CLocalPath local_path(path);
+	if (!local_path.HasParent() || !local_path.IsWriteable())
 		return;
 
-#ifdef __WXMSW__
-	if (path.Len() == 3 && path.Mid(1) == _T(":\\"))
-		return;
-#endif
-
 	if (!CLocalFileSystem::RecursiveDelete(path, this))
 		wxGetApp().DisplayEncodingWarning();
 
@@ -1411,17 +1407,9 @@ void CLocalTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 	}
 #endif
 
-	wxString path = GetDirFromItem(item);
+	CLocalPath path(GetDirFromItem(item));
 
-#ifdef __WXMSW__
-	if (path.Len() == 3 && path.Mid(1) == _T(":\\"))
-	{
-		wxBell();
-		event.Veto();
-		return;
-	}
-#endif
-	if (!m_pState->LocalDirHasParent(path) || !m_pState->LocalDirIsWriteable(path))
+	if (!path.HasParent() || !path.IsWriteable())
 	{
 		wxBell();
 		event.Veto();
@@ -1453,15 +1441,8 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 
 	wxString path = GetDirFromItem(item);
 
-#ifdef __WXMSW__
-	if (path.Len() == 3 && path.Mid(1) == _T(":\\"))
-	{
-		wxBell();
-		event.Veto();
-		return;
-	}
-#endif
-	if (!m_pState->LocalDirHasParent(path) || !m_pState->LocalDirIsWriteable(path))
+	CLocalPath local_path(path);
+	if (!local_path.HasParent() || !local_path.IsWriteable())
 	{
 		wxBell();
 		event.Veto();
@@ -1496,6 +1477,12 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		return;
 	}
 
+	// We may call SetLocalDir, item might be deleted by it, so
+	// if we don't rename item now and veto the event, wxWidgets
+	// might access deleted item.
+	event.Veto();
+	SetItemText(item, newName);
+
 	wxTreeItemId currentSel = GetSelection();
 	if (currentSel == wxTreeItemId())
 	{
@@ -1540,3 +1527,191 @@ void CLocalTreeView::OnChar(wxKeyEvent& event)
 	else
 		event.Skip();
 }
+
+bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
+{
+	wxTreeItemIdValue value;
+	wxTreeItemId child = GetFirstChild(item, value);
+
+	static const wxLongLong size(-1);
+
+#ifdef __WXMAC__
+	// By default, OS X has a list of servers mounted into /net,
+	// listing that directory is slow.
+	if (GetItemParent(item) == GetRootItem() && (path == _T("/net") || path == _T("/net/")))
+	{
+			CFilterManager filter;
+
+			const int attributes = S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
+			if (!filter.FilenameFiltered(_T("localhost"), path, true, size, true, attributes))
+			{
+				if (!child)
+					AppendItem(item, _T(""));
+				return true;
+			}
+	}
+#endif
+
+	if (child)
+	{
+		if (GetItemText(child) != _T(""))
+			return false;
+
+		CTreeItemData* pData = (CTreeItemData*)GetItemData(child);
+		if (pData)
+		{
+			bool wasLink;
+			int attributes;
+			enum CLocalFileSystem::local_fileType type;
+			if (path.Last() == CLocalFileSystem::path_separator)
+				type = CLocalFileSystem::GetFileInfo(path + pData->m_known_subdir, wasLink, 0, 0, &attributes);
+			else
+				type = CLocalFileSystem::GetFileInfo(path + CLocalFileSystem::path_separator + pData->m_known_subdir, wasLink, 0, 0, &attributes);
+			if (type == CLocalFileSystem::dir)
+			{
+				CFilterManager filter;
+				if (!filter.FilenameFiltered(pData->m_known_subdir, path, true, size, true, attributes))
+					return true;
+			}
+		}
+	}
+
+	wxString sub = HasSubdir(path);
+	if (sub != _T(""))
+	{
+		wxTreeItemId subItem = AppendItem(item, _T(""));
+		SetItemData(subItem, new CTreeItemData(sub));
+	}
+	else if (child)
+		Delete(child);
+
+	return true;
+}
+
+#ifdef __WXMSW__
+void CLocalTreeView::OnDevicechange(WPARAM wParam, LPARAM lParam)
+{
+	if (!m_drives)
+		return;
+
+	if (wParam == DBT_DEVICEARRIVAL || wParam == DBT_DEVICEREMOVECOMPLETE)
+	{
+		DEV_BROADCAST_HDR* pHdr = (DEV_BROADCAST_HDR*)lParam;
+		if (pHdr->dbch_devicetype != DBT_DEVTYP_VOLUME)
+			return;
+
+		// Added or removed volume
+		DEV_BROADCAST_VOLUME* pVolume = (DEV_BROADCAST_VOLUME*)lParam;
+
+		wxChar drive = 'A';
+		int mask = 1;
+		while (drive <= 'Z')
+		{
+			if (pVolume->dbcv_unitmask & mask)
+			{
+				if (wParam == DBT_DEVICEARRIVAL)
+					AddDrive(drive);
+				else
+				{
+					RemoveDrive(drive);
+
+					if (pVolume->dbcv_flags & DBTF_MEDIA)
+					{
+						// E.g. disk removed from CD-ROM drive, need to keep the drive letter
+						AddDrive(drive);
+					}
+				}
+			}
+			drive++;
+			mask <<= 1;
+		}
+
+		if (GetSelection() == m_drives && m_pState)
+			m_pState->RefreshLocal();
+	}
+}
+
+wxTreeItemId CLocalTreeView::AddDrive(wxChar drive)
+{
+	// Adhere to the NODRIVES group policy
+	long drivesToHide = 0;
+	wxRegKey key(_T("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer"));
+	if (key.Exists())
+	{
+		if (!key.HasValue(_T("NoDrives")) || !key.QueryValue(_T("NoDrives"), &drivesToHide))
+			drivesToHide = 0;
+	}
+
+	int bit = 0;
+	if (drive >= 'A' && drive <= 'Z')
+		bit = 1 << (drive - 'A');
+	else if (drive >= 'a' && drive <= 'z')
+		bit = 1 << (drive - 'a');
+	if (drivesToHide & bit)
+		return wxTreeItemId();
+
+	wxString driveName = drive;
+	driveName += _T(":");
+
+	// Get the label of the drive
+	wxChar volumeName[501];
+	int oldErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
+	BOOL res = GetVolumeInformation(driveName + _T("\\"), volumeName, 500, 0, 0, 0, 0, 0);
+	SetErrorMode(oldErrorMode);
+
+	wxString itemLabel = driveName;
+	if (res && volumeName[0])
+	{
+		itemLabel += _T(" (");
+		itemLabel += volumeName;
+		itemLabel += _T(")");
+	}
+
+	wxTreeItemIdValue value;
+	wxTreeItemId driveItem = GetFirstChild(m_drives, value);
+	while (driveItem)
+	{
+		if (!GetItemText(driveItem).Left(2).CmpNoCase(driveName))
+			break;
+
+		driveItem = GetNextSibling(driveItem);
+	}
+	if (driveItem)
+	{
+		SetItemText(driveItem, itemLabel);
+		int icon = GetIconIndex(dir, driveName + _T("\\"));
+		SetItemImage(driveItem, icon, wxTreeItemIcon_Normal);
+		SetItemImage(driveItem, icon, wxTreeItemIcon_Selected);
+		SetItemImage(driveItem, icon, wxTreeItemIcon_Expanded);
+		SetItemImage(driveItem, icon, wxTreeItemIcon_SelectedExpanded);
+
+		return driveItem;
+	}
+
+	wxTreeItemId item = AppendItem(m_drives, itemLabel, GetIconIndex(dir, driveName + _T("\\")));
+	AppendItem(item, _T(""));
+	SortChildren(m_drives);
+
+	return item;
+}
+
+void CLocalTreeView::RemoveDrive(wxChar drive)
+{
+	wxString driveName = drive;
+	driveName += _T(":");
+	wxTreeItemIdValue value;
+	wxTreeItemId driveItem = GetFirstChild(m_drives, value);
+	while (driveItem)
+	{
+		if (!GetItemText(driveItem).Left(2).CmpNoCase(driveName))
+			break;
+
+		driveItem = GetNextSibling(driveItem);
+	}
+	if (!driveItem)
+		return;
+
+	Delete(driveItem);
+}
+
+#endif
