@@ -972,7 +972,10 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			{
 				const CServer* pServer = m_pState->GetServer();
 				if (!pServer || *pServer != pData->m_server)
-					m_pState->Connect(pData->m_server, true, pData->m_remoteDir);
+				{
+					ConnectToSite(pData);
+					pData->m_localDir.clear(); // So not to set again below
+				}
 				else
 					m_pState->ChangeRemoteDir(pData->m_remoteDir);
 			}
@@ -1160,6 +1163,14 @@ void CMainFrame::OnEngineEvent(wxEvent &event)
 				delete pNotification;
 			}
 			break;
+		case nId_local_dir_created:
+			if (m_pState)
+			{
+				CLocalDirCreatedNotification *pLocalDirCreatedNotification = reinterpret_cast<CLocalDirCreatedNotification *>(pNotification);
+				m_pState->LocalDirCreated(pLocalDirCreatedNotification->dir);
+			}
+			delete pNotification;
+			break;
 		default:
 			delete pNotification;
 			break;
@@ -1588,7 +1599,8 @@ void CMainFrame::SetProgress(const CTransferStatus *pStatus)
 void CMainFrame::OpenSiteManager(const CServer* pServer /*=0*/)
 {
 	CSiteManager dlg;
-	if (!dlg.Create(this, pServer))
+
+	if (!dlg.Create(this, m_last_bookmark_path, pServer))
 		return;
 
 	int res = dlg.ShowModal();
@@ -1603,6 +1615,9 @@ void CMainFrame::OpenSiteManager(const CServer* pServer /*=0*/)
 			m_last_bookmark_path = dlg.GetSitePath();
 		}
 	}
+	else if (res == wxID_OK)
+		m_last_bookmark_path = dlg.GetChangedBookmarkPath(m_pState->GetServer());
+
 	m_bookmarks.clear();
 	dlg.GetBookmarks(m_last_bookmark_path, m_bookmarks);
 	UpdateBookmarkMenu();
