@@ -60,6 +60,12 @@ enum interfaceOptions
 	OPTION_REMOTEFILELIST_COLUMN_ORDER,
 	OPTION_FILELIST_STATUSBAR,
 	OPTION_FILTERTOGGLESTATE,
+	OPTION_SIZE_DECIMALPLACES,
+	OPTION_SHOW_QUICKCONNECT,
+	OPTION_MESSAGELOG_POSITION,
+	OPTION_LAST_CONNECTED_SITE,
+	OPTION_DOUBLECLICK_ACTION_FILE,
+	OPTION_DOUBLECLICK_ACTION_DIRECTORY,
 
 	// Has to be last element
 	OPTIONS_NUM
@@ -84,7 +90,7 @@ struct t_OptionsCache
 
 class CXmlFile;
 class TiXmlElement;
-class COptions : public COptionsBase
+class COptions : public wxEvtHandler, public COptionsBase
 {
 public:
 	virtual int GetOptionVal(unsigned int nID);
@@ -96,9 +102,6 @@ public:
 	virtual bool SetOption(unsigned int nID, int value);
 	virtual bool SetOption(unsigned int nID, wxString value);
 	
-	bool SetOptionNoSave(unsigned int nID, int value);
-	bool SetOptionNoSave(unsigned int nID, wxString value);
-
 	void SetLastServer(const CServer& server);
 	bool GetLastServer(CServer& server);
 
@@ -107,7 +110,9 @@ public:
 	static void Destroy();
 
 	void Import(TiXmlElement* pElement);
-	
+
+	void SaveIfNeeded();
+
 protected:
 	COptions();
 	virtual ~COptions();
@@ -126,6 +131,8 @@ protected:
 
 	void LoadGlobalDefaultOptions();
 
+	void Save();
+
 	CXmlFile* m_pXmlFile;
 
 	t_OptionsCache m_optionsCache[OPTIONS_NUM];
@@ -134,6 +141,11 @@ protected:
 	CServer* m_pLastServer;
 
 	static COptions* m_theOptions;
+
+	wxTimer m_save_timer;
+
+	DECLARE_EVENT_TABLE();
+	void OnTimer(wxTimerEvent& event);
 };
 
 #endif //__OPTIONS_H__
