@@ -5,12 +5,28 @@
 #include "dndobjects.h"
 #include <wx/progdlg.h>
 
-struct t_newEntry
+class CFolderProcessingEntry
 {
-	wxString localFile;
-	wxString remoteFile;
-	CServerPath remotePath;
+public:
+	enum t_type
+	{
+		dir,
+		file
+	};
+	const enum t_type m_type;
+
+	CFolderProcessingEntry(enum t_type type) : m_type(type) {}
+};
+
+class t_newEntry : public CFolderProcessingEntry
+{
+public:
+	t_newEntry() : CFolderProcessingEntry(CFolderProcessingEntry::file) {}
+	wxString name;
 	wxLongLong size;
+	wxDateTime time;
+	int attributes;
+	bool dir;
 };
 
 enum ActionAfterState
@@ -20,15 +36,17 @@ enum ActionAfterState
 	ActionAfterState_Disconnect,
 	ActionAfterState_RunCommand,
 	ActionAfterState_ShowMessage,
-	ActionAfterState_PlaySound,
+	ActionAfterState_PlaySound
 // On Windows, wx can reboot or shutdown the system as well.
 #ifdef __WXMSW__
+	,
 	ActionAfterState_Reboot,
 	ActionAfterState_Shutdown
 #endif
 };
 
 class CStatusLineCtrl;
+class CFileItem;
 struct t_EngineData
 {
 	CFileZillaEngine* pEngine;
@@ -71,7 +89,7 @@ public:
 		const CServerPath& remotePath, const CServer& server, const wxLongLong size, enum CEditHandler::fileType edit = CEditHandler::none);
 	void QueueFile_Finish(const bool start); // Need to be called after QueueFile
 	bool QueueFiles(const bool queueOnly, const wxString& localPath, const CRemoteDataObject& dataObject);
-	bool QueueFiles(const std::list<t_newEntry*> &entryList, bool queueOnly, bool download, CServerItem* pServerItem, const enum CFileExistsNotification::OverwriteAction defaultFileExistsAction);
+	int QueueFiles(const std::list<CFolderProcessingEntry*> &entryList, bool queueOnly, bool download, CServerItem* pServerItem, const enum CFileExistsNotification::OverwriteAction defaultFileExistsAction);
 	bool QueueFolder(bool queueOnly, bool download, const wxString& localPath, const CServerPath& remotePath, const CServer& server);
 
 	bool IsEmpty() const;
@@ -87,11 +105,6 @@ public:
 	// This includes queued folders which are yet to be processed
 	void SetDefaultFileExistsAction(enum CFileExistsNotification::OverwriteAction action, const enum TransferDirection direction);
 
-	// Tries to refresh the current remote directory listing
-	// if there's an idle engine connected to the current server of
-	// the primary connection.
-	void TryRefreshListings();
-
 	void UpdateItemSize(CFileItem* pItem, wxLongLong size);
 
 	void RemoveAll();
@@ -109,7 +122,7 @@ public:
 
 protected:
 
-	void AdvanceQueue();
+	void AdvanceQueue(bool refresh = true);
 	bool TryStartNextTransfer();
 	bool ProcessFolderItems(int type = -1);
 	void ProcessUploadFolderItems();
@@ -144,7 +157,6 @@ protected:
 	void CalculateQueueSize();
 	void DisplayQueueSize();
 	void SaveQueue();
-	bool ShouldUseBinaryMode(wxString filename, enum ServerType type);
 
 	bool IsActionAfter(enum ActionAfterState);
 	void ActionAfter(bool warned = false);
@@ -154,6 +166,14 @@ protected:
 
 	void ProcessNotification(t_EngineData* pEngineData, CNotification* pNotification);
 
+	// Tries to refresh the current remote directory listing
+	// if there's an idle engine connected to the current server of
+	// the primary connection.
+	void TryRefreshListings();
+	CServer m_last_refresh_server;
+	CServerPath m_last_refresh_path;
+	CTimeEx m_last_refresh_listing_time;
+
 	// Called from Process Reply.
 	// After a disconnect, check if there's another idle engine that
 	// is already connected.
@@ -174,7 +194,7 @@ protected:
 	std::list<CFolderScanItem*> m_queuedFolders[2];
 
 	CFolderProcessingThread *m_pFolderProcessingThread;
-
+	
 	/*
 	 * Don't update status line positions if m_waitStatusLineUpdate is true.
 	 * This assures we are updating the status line positions only once,
@@ -206,8 +226,6 @@ protected:
 	CMainFrame* m_pMainFrame;
 	CAsyncRequestQueue* m_pAsyncRequestQueue;
 
-	std::list<wxString> m_asciiFiles;
-
 	std::list<CFileZillaEngine*> m_waitingForPassword;
 
 	virtual void OnPostScroll();
