@@ -19,7 +19,7 @@ public:
 	CRemoteListView(wxWindow* pParent, CState* pState, CQueueView* pQueue);
 	virtual ~CRemoteListView();
 
-	bool DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, wxString path, bool queueOnly);
+	bool DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, const CLocalPath& path, bool queueOnly);
 
 	void InitDateFormat();
 
@@ -29,6 +29,8 @@ public:
 	virtual void FinishComparison();
 	virtual void OnExitComparisonMode();
 
+	void LinkIsNotDir(const CServerPath& path, const wxString& link);
+
 protected:
 	virtual wxString GetItemText(int item, unsigned int column);
 
@@ -62,6 +64,7 @@ protected:
 	void SetDirectoryListing(const CDirectoryListing *pDirectoryListing, bool modified = false);
 	bool UpdateDirectoryListing(const CDirectoryListing *pDirectoryListing);
 	void UpdateDirectoryListing_Removed(const CDirectoryListing *pDirectoryListing);
+	void UpdateDirectoryListing_Added(const CDirectoryListing *pDirectoryListing);
 
 #ifdef __WXDEBUG__
 	void ValidateIndexMapping();
@@ -70,7 +73,7 @@ protected:
 	const CDirectoryListing *m_pDirectoryListing;
 
 	// Caller is responsible to check selection is valid!
-	void TransferSelectedFiles(const wxString& localDir, bool queueOnly);
+	void TransferSelectedFiles(const CLocalPath& local_parent, bool queueOnly);
 
 	// Cache icon for directories, no need to calculate it multiple times
 	int m_dirIcon;
@@ -85,6 +88,17 @@ protected:
 	wxString m_dateFormat;
 	wxString m_timeFormat;
 
+	// Used to track state for resolving symlinks
+	// While being resolved, global state might have changed
+	// already.
+	struct t_linkResolveState
+	{
+		CServer server;
+		CServerPath remote_path;
+		wxString link;
+		wxString local_path;
+	} *m_pLinkResolveState;
+
 	DECLARE_EVENT_TABLE()
 	void OnItemActivated(wxListEvent &event);
 	void OnContextMenu(wxContextMenuEvent& event);
@@ -92,7 +106,6 @@ protected:
 	void OnMenuMkdir(wxCommandEvent& event);
 	void OnMenuDelete(wxCommandEvent& event);
 	void OnMenuRename(wxCommandEvent& event);
-	void OnChar(wxKeyEvent& event);
 	void OnKeyDown(wxKeyEvent& event);
 	void OnBeginLabelEdit(wxListEvent& event);
 	void OnEndLabelEdit(wxListEvent& event);
@@ -100,6 +113,7 @@ protected:
 	void OnSize(wxSizeEvent& event);
 	void OnBeginDrag(wxListEvent& event);
 	void OnMenuEdit(wxCommandEvent& event);
+	void OnMenuEnter(wxCommandEvent& event);
 };
 
 #endif
