@@ -6,6 +6,7 @@
 #include "chmoddialog.h"
 #include "recursive_operation.h"
 #include "inputdialog.h"
+#include "dragdropmanager.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -70,6 +71,10 @@ public:
 		if (!GetData())
 			return wxDragError;
 
+		CDragDropManager* pDragDropManager = CDragDropManager::Get();
+		if (pDragDropManager)
+			pDragDropManager->pDropTarget = m_pRemoteTreeView;
+
 		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
 			m_pRemoteTreeView->m_pState->UploadDroppedFiles(m_pFileDataObject, path, false);
 		else
@@ -229,6 +234,10 @@ CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState
 	CSystemImageList(16),
 	CStateEventHandler(pState)
 {
+#ifdef __WXMAC__
+	SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
+#endif
+
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR_MODIFIED);
 	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
@@ -266,7 +275,7 @@ void CRemoteTreeView::SetDirectoryListing(const CDirectoryListing* pListing, boo
 
 	if (!pListing)
 	{
-        m_ExpandAfterList = wxTreeItemId();
+		m_ExpandAfterList = wxTreeItemId();
 		DeleteAllItems();
 		AddRoot(_T(""));
 		m_busy = false;
@@ -291,10 +300,16 @@ void CRemoteTreeView::SetDirectoryListing(const CDirectoryListing* pListing, boo
 		return;
 	}
 
+#ifndef __WXMSW__
+	Freeze();
+#endif
 	wxTreeItemId parent = MakeParent(pListing->path, !modified);
 	if (!parent)
 	{
 		m_busy = false;
+#ifndef __WXMSW__
+		Thaw();
+#endif
 		return;
 	}
 
@@ -307,16 +322,34 @@ void CRemoteTreeView::SetDirectoryListing(const CDirectoryListing* pListing, boo
 	}
 	else
 	{
-		RefreshItem(parent, *pListing);
+		RefreshItem(parent, *pListing, !modified);
 
 		if (m_ExpandAfterList == parent)
+		{
+#ifndef __WXMSW__
+			// Prevent CalculatePositions from being called
+			wxGenericTreeItem *anchor = m_anchor;
+			m_anchor = 0;
+#endif
 			Expand(parent);
+#ifndef __WXMSW__
+			m_anchor = anchor;
+#endif
+		}
 	}
 	m_ExpandAfterList = wxTreeItemId();
 
 	SetItemImages(parent, false);
 
-	Refresh(false);
+#ifndef __WXMSW__
+	m_freezeCount--;
+#endif
+	if (!modified)
+		SelectItem(parent);
+#ifndef __WXMSW__
+	else
+		Refresh();
+#endif
 
 	m_busy = false;
 }
@@ -337,25 +370,23 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 
 	for (std::list<wxString>::const_iterator iter = pieces.begin(); iter != pieces.end(); iter++)
 	{
-		if (parent != root)
-			ListExpand(parent);
-
 		if (iter != pieces.begin())
 			path.AddSegment(*iter);
 
 		wxTreeItemIdValue cookie;
-		wxTreeItemId child;
+		wxTreeItemId child = GetFirstChild(parent, cookie);
+		if (child && GetItemText(child) == _T(""))
+		{
+			Delete(child);
+			child = wxTreeItemId();
+			if (parent != root)
+				ListExpand(parent);
+		}
 		for (child = GetFirstChild(parent, cookie); child; child = GetNextSibling(child))
 		{
 			const wxString& text = GetItemText(child);
 			if (text == *iter)
 				break;
-			if (text == _T(""))
-			{
-				Delete(child);
-				child = wxTreeItemId();
-				break;
-			}
 		}
 		if (!child)
 		{
@@ -363,12 +394,12 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 
 			if (m_pState->m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK)
 			{
-				child = AppendItem(parent, *iter, 0, 2, new CItemData(path));
+				child = AppendItem(parent, *iter, 0, 2, path.HasParent() ? 0 : new CItemData(path));
 				SetItemImages(child, false);
 			}
 			else
 			{
-				child = AppendItem(parent, *iter, 1, 3, new CItemData(path));
+				child = AppendItem(parent, *iter, 1, 3, path.HasParent() ? 0 : new CItemData(path));
 				SetItemImages(child, true);
 			}
 			SortChildren(parent);
@@ -379,12 +410,20 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 				DisplayItem(child, listing);
 		}
 		if (select && iter != pieces.begin())
+		{
+#ifndef __WXMSW__
+			// Prevent CalculatePositions from being called
+			wxGenericTreeItem *anchor = m_anchor;
+			m_anchor = 0;
+#endif
 			Expand(parent);
+#ifndef __WXMSW__
+			m_anchor = anchor;
+#endif
+		}
 
 		parent = child;
 	}
-	if (select)
-		SelectItem(parent);
 
 	return parent;
 }
@@ -488,7 +527,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 
 		if (m_pState->m_pEngine->CacheLookup(subdir, subListing) == FZ_REPLY_OK)
 		{
-			wxTreeItemId child = AppendItem(parent, name, 0, 2, new CItemData(subdir));
+			wxTreeItemId child = AppendItem(parent, name, 0, 2, 0);
 			SetItemImages(child, false);
 
 			if (HasSubdirs(subListing, filter))
@@ -496,7 +535,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 		}
 		else
 		{
-			wxTreeItemId child = AppendItem(parent, name, 1, 3, new CItemData(subdir));
+			wxTreeItemId child = AppendItem(parent, name, 1, 3, 0);
 			SetItemImages(child, true);
 		}
 	}
@@ -514,7 +553,7 @@ static bool sortfunc(const wxString& a, const wxString& b)
 }
 
 
-void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing& listing)
+void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing& listing, bool will_select_parent)
 {
 	SetItemImages(parent, false);
 
@@ -576,7 +615,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			while (sel && sel != child)
 				sel = GetItemParent(sel);
 			wxTreeItemId prev = GetPrevSibling(child);
-			if (!sel)
+			if (!sel || will_select_parent)
 				Delete(child);
 			child = prev;
 		}
@@ -589,7 +628,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			CDirectoryListing subListing;
 			if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK)
 			{
-				wxTreeItemId child = AppendItem(parent, *iter, 0, 2, new CItemData(path));
+				wxTreeItemId child = AppendItem(parent, *iter, 0, 2, 0);
 				SetItemImages(child, false);
 
 				if (HasSubdirs(subListing, filter))
@@ -597,9 +636,9 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			}
 			else
 			{
-				wxTreeItemId child = AppendItem(parent, *iter, 1, 3, new CItemData(path));
-                if (child)
-				    SetItemImages(child, true);
+				wxTreeItemId child = AppendItem(parent, *iter, 1, 3, 0);
+				if (child)
+					SetItemImages(child, true);
 			}
 
 			iter++;
@@ -613,7 +652,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		while (sel && sel != child)
 			sel = GetItemParent(sel);
 		wxTreeItemId prev = GetPrevSibling(child);
-		if (!sel)			
+		if (!sel || will_select_parent)
 			Delete(child);
 		child = prev;
 	}
@@ -625,7 +664,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		CDirectoryListing subListing;
 		if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK)
 		{
-			wxTreeItemId child = AppendItem(parent, *iter, 0, 2, new CItemData(path));
+			wxTreeItemId child = AppendItem(parent, *iter, 0, 2, 0);
 			SetItemImages(child, false);
 
 			if (HasSubdirs(subListing, filter))
@@ -633,7 +672,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		}
 		else
 		{
-			wxTreeItemId child = AppendItem(parent, *iter, 1, 3, new CItemData(path));
+			wxTreeItemId child = AppendItem(parent, *iter, 1, 3, 0);
 			SetItemImages(child, true);
 		}
 
@@ -676,8 +715,11 @@ void CRemoteTreeView::OnItemExpanding(wxTreeEvent& event)
 
 void CRemoteTreeView::SetItemImages(wxTreeItemId item, bool unknown)
 {
+	const int old_image = GetItemImage(item, wxTreeItemIcon_Normal);
 	if (!unknown)
 	{
+		if (old_image == 0)
+			return;
 		SetItemImage(item, 0, wxTreeItemIcon_Normal);
 		SetItemImage(item, 2, wxTreeItemIcon_Selected);
 		SetItemImage(item, 0, wxTreeItemIcon_Expanded);
@@ -685,6 +727,8 @@ void CRemoteTreeView::SetItemImages(wxTreeItemId item, bool unknown)
 	}
 	else
 	{
+		if (old_image == 1)
+			return;
 		SetItemImage(item, 1, wxTreeItemIcon_Normal);
 		SetItemImage(item, 3, wxTreeItemIcon_Selected);
 		SetItemImage(item, 1, wxTreeItemIcon_Expanded);
@@ -709,12 +753,12 @@ void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 	if (!item)
 		return;
 
-	const CItemData* data = (CItemData*)GetItemData(item);
-	wxASSERT(data);
-	if (!data)
+	const CServerPath path = GetPathFromItem(item);
+	wxASSERT(!path.IsEmpty());
+	if (path.IsEmpty())
 		return;
 
-	m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(data->m_path));
+	m_pState->ChangeRemoteDir(path);
 }
 
 void CRemoteTreeView::OnItemActivated(wxTreeEvent& event)
@@ -725,12 +769,28 @@ void CRemoteTreeView::OnItemActivated(wxTreeEvent& event)
 
 CServerPath CRemoteTreeView::GetPathFromItem(const wxTreeItemId& item) const
 {
-	const CItemData* const pData = (const CItemData*)GetItemData(item);
+	std::list<wxString> segments;
 
-	if (!pData)
-		return CServerPath();
+	wxTreeItemId i = item;
+	while (i != GetRootItem())
+	{
+		const CItemData* const pData = (const CItemData*)GetItemData(i);
+		if (pData)
+		{
+			CServerPath path = pData->m_path;
+			for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); iter++)
+			{
+				if (!path.AddSegment(*iter))
+					return CServerPath();
+			}
+			return path;
+		}
 
-	return pData->m_path;
+		segments.push_front(GetItemText(i));
+		i = GetItemParent(i);
+	}
+
+	return CServerPath();
 }
 
 void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
@@ -762,7 +822,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 		return;
 
 	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(*pServer, parent);
-	pRemoteDataObject->AddFile(lastSegment, true, -1);
+	pRemoteDataObject->AddFile(lastSegment, true, -1, false);
 
 	pRemoteDataObject->Finalize();
 
@@ -783,9 +843,19 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 	}
 #endif
 
+	CDragDropManager* pDragDropManager = CDragDropManager::Init();
+	pDragDropManager->pDragSource = this;
+	pDragDropManager->server = *pServer;
+	pDragDropManager->remoteParent = parent;
+
 	wxDropSource source(this);
 	source.SetData(object);
-	if (source.DoDragDrop() != wxDragCopy)
+
+	int res = source.DoDragDrop();
+
+	pDragDropManager->Release();
+
+	if (res != wxDragCopy)
 	{
 #if FZ3_USESHELLEXT
 		delete ext;
@@ -810,8 +880,8 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 				return;
 			}
 
-			wxString target = ext->GetTarget();
-			if (target == _T(""))
+			CLocalPath target(ext->GetTarget());
+			if (target.empty())
 			{
 				delete ext;
 				ext = 0;
@@ -870,8 +940,7 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 	else if (!path.HasParent())
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 
-	const wxString& localDir = m_pState->GetLocalDir();
-	if (!m_pState->LocalDirIsWriteable(localDir))
+	if (!m_pState->GetLocalDir().IsWriteable())
 	{
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
@@ -987,15 +1056,15 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent& event)
 		if (selected)
 		{
 			CServerPath currentPath = GetPathFromItem(selected);
-			m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(currentPath));
+			m_pState->ChangeRemoteDir(currentPath);
 		}
 	}
 }
 
 void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 {
-	const wxString& localDir = m_pState->GetLocalDir();
-	if (!m_pState->LocalDirIsWriteable(localDir))
+	CLocalPath localDir = m_pState->GetLocalDir();
+	if (!localDir.IsWriteable())
 	{
 		wxBell();
 		return;
@@ -1011,17 +1080,12 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 	if (path.IsEmpty())
 		return;
 
-	const bool hasParent = path.HasParent();
-
 	const wxString& name = GetItemText(m_contextMenuItem);
 
-	wxFileName fn = wxFileName(localDir, _T(""));
-	
-	if (hasParent)
-		fn.AppendDir(name);	
+	localDir.AddSegment(name);
 
 	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
-	pRecursiveOperation->AddDirectoryToVisit(path, _T(""), fn.GetFullPath());
+	pRecursiveOperation->AddDirectoryToVisit(path, _T(""), localDir.GetPath());
 
 	CServerPath currentPath;
 	const wxTreeItemId selected = GetSelection();
@@ -1129,26 +1193,14 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	}
 
 	CItemData* const pData = (CItemData*)GetItemData(event.GetItem());
-	if (!pData)
+	if (pData)
 	{
 		event.Veto();
 		return;
 	}
 
-	const CServerPath path = pData->m_path;
-	if (path.IsEmpty())
-	{
-		event.Veto();
-		return;
-	}
-
-	if (!path.HasParent())
-	{
-		event.Veto();
-		return;
-	}
-
-	CServerPath parent = path.GetParent();
+	CServerPath old_path = GetPathFromItem(event.GetItem());
+	CServerPath parent = old_path.GetParent();
 
 	const wxString& oldName = GetItemText(event.GetItem());
 	const wxString& newName = event.GetLabel();
@@ -1158,13 +1210,8 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		return;
 	}
 
-	// Update the item data
-	wxASSERT(path.GetLastSegment() == oldName);
-	pData->m_path = parent;
-	pData->m_path.AddSegment(newName);
-
 	m_pState->m_pCommandQueue->ProcessCommand(new CRenameCommand(parent, oldName, parent, newName));
-	m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(parent));
+	m_pState->ChangeRemoteDir(parent);
 
 	CServerPath currentPath;
 	const wxTreeItemId selected = GetSelection();
@@ -1173,12 +1220,12 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	if (currentPath.IsEmpty())
 		return;
 
-	if (currentPath == path || currentPath.IsSubdirOf(path, false))
+	if (currentPath == old_path || currentPath.IsSubdirOf(old_path, false))
 	{
 		// Previously selected path was below renamed one, list the new one
 
 		std::list<wxString> subdirs;
-		while (currentPath != path)
+		while (currentPath != old_path)
 		{
 			if (!currentPath.HasParent())
 			{
@@ -1192,10 +1239,10 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		currentPath.AddSegment(newName);
 		for (std::list<wxString>::const_iterator iter = subdirs.begin(); iter != subdirs.end(); iter++)
 			currentPath.AddSegment(*iter);
-		m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(currentPath));
+		m_pState->ChangeRemoteDir(currentPath);
 	}
 	else if (currentPath != parent)
-		m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(currentPath));
+		m_pState->ChangeRemoteDir(currentPath);
 }
 
 void CRemoteTreeView::OnMkdir(wxCommandEvent& event)
@@ -1217,8 +1264,8 @@ void CRemoteTreeView::OnMkdir(wxCommandEvent& event)
 	CServerPath newPath = path;
 
 	// Append a long segment which does (most likely) not exist in the path and
-	// replace it with "New folder" later. This way we get the exact position of
-	// "New folder" and can preselect it in the dialog.
+	// replace it with "New directory" later. This way we get the exact position of
+	// "New directory" and can preselect it in the dialog.
 	wxString tmpName = _T("25CF809E56B343b5A12D1F0466E3B37A49A9087FDCF8412AA9AF8D1E849D01CF");
 	if (newPath.AddSegment(tmpName))
 	{
@@ -1246,7 +1293,7 @@ void CRemoteTreeView::OnMkdir(wxCommandEvent& event)
 	if (newPath.HasParent())
 	{
 		listed = newPath.GetParent();
-		m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(listed));
+		m_pState->ChangeRemoteDir(listed);
 	}
 
 	CServerPath currentPath;
@@ -1254,19 +1301,19 @@ void CRemoteTreeView::OnMkdir(wxCommandEvent& event)
 	if (selected)
 		currentPath = GetPathFromItem(selected);
 	if (!currentPath.IsEmpty() && currentPath != listed)
-		m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(currentPath));
+		m_pState->ChangeRemoteDir(currentPath);
 }
 
 bool CRemoteTreeView::ListExpand(wxTreeItemId item)
 {
-	const CItemData* data = (CItemData*)GetItemData(item);
-	wxASSERT(data);
-	if (!data)
+	const CServerPath path = GetPathFromItem(item);
+	wxASSERT(!path.IsEmpty());
+	if (path.IsEmpty())
 		return false;
 
 	CDirectoryListing listing;
-	if (m_pState->m_pEngine->CacheLookup(data->m_path, listing) == FZ_REPLY_OK)
-		RefreshItem(item, listing);
+	if (m_pState->m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK)
+		RefreshItem(item, listing, false);
 	else
 	{
 		SetItemImages(item, true);
