@@ -29,6 +29,7 @@ EVT_COMMAND(wxID_ANY, fz_EVT_FILELIST_FOCUSCHANGE, CFileListCtrl<CFileData>::OnP
 EVT_LEFT_DOWN(CFileListCtrl<CFileData>::OnLeftDown)
 EVT_COMMAND(wxID_ANY, fz_EVT_DEFERRED_MOUSEEVENT, CFileListCtrl<CFileData>::OnProcessMouseEvent)
 #endif
+EVT_KEY_DOWN(CFileListCtrl<CFileData>::OnKeyDown)
 END_EVENT_TABLE()
 
 #ifdef __WXMSW__
@@ -386,7 +387,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnClicked(wxListE
 		dir = m_sortDirection;
 
 	SortList(col, dir);
-	Refresh(false);
+	RefreshListOnly(false);
 }
 
 template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString name, bool dir, const wxString& path /*=_T("")*/)
@@ -449,14 +450,15 @@ template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString na
 	return type;
 #else
 	if (dir)
-		return _("Folder");
+		return _("Directory");
 
-	wxFileName fn(name);
-	wxString ext = fn.GetExt();
-	if (ext == _T(""))
+	int pos = name.Find('.', true);
+	if (pos < 1 || !name[pos + 1]) // Starts or ends with dot
 		return _("File");
+	wxString ext = name.Mid(pos + 1);
+	wxString lower_ext = ext.Lower();
 
-	std::map<wxString, wxString>::iterator typeIter = m_fileTypeMap.find(ext);
+	std::map<wxString, wxString>::iterator typeIter = m_fileTypeMap.find(lower_ext);
 	if (typeIter != m_fileTypeMap.end())
 		return typeIter->second;
 
@@ -482,7 +484,7 @@ template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString na
 	desc = ext;
 	desc += _T("-");
 	desc += _("file");
-	m_fileTypeMap[ext.MakeLower()] = desc;
+	m_fileTypeMap[lower_ext] = desc;
 	return desc;
 #endif
 }
@@ -519,7 +521,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnExitComparisonMode()
 
 	ComparisonRestoreSelections();
 
-	Refresh();
+	RefreshListOnly();
 }
 
 template<class CFileData> void CFileListCtrl<CFileData>::CompareAddFile(t_fileEntryFlags flags)
@@ -874,3 +876,24 @@ template<class CFileData> void CFileListCtrl<CFileData>::ClearSelection()
 	if (item != -1)
 		SetItemState(item, 0, wxLIST_STATE_FOCUSED);
 }
+
+template<class CFileData> void CFileListCtrl<CFileData>::OnKeyDown(wxKeyEvent& event)
+{
+	const int code = event.GetKeyCode();
+	const int mods = event.GetModifiers();
+	if (code == 'A' && (mods == wxMOD_CMD || mods == (wxMOD_CONTROL | wxMOD_META)))
+	{
+		for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); i++)
+		{
+			const CFileData& data = m_fileData[m_indexMapping[i]];
+			if (data.flags != fill)
+				SetSelection(i, true);
+			else
+				SetSelection(i, false);
+		}
+		if (m_pFilelistStatusBar)
+			m_pFilelistStatusBar->SelectAll();
+	}
+	else
+		event.Skip();
+}
