@@ -133,6 +133,114 @@ COptionsPageThemes::~COptionsPageThemes()
 }
 
 bool COptionsPageThemes::LoadPage()
+{
+	return true;
+}
+
+bool COptionsPageThemes::SavePage()
+{
+	if (!m_was_selected)
+		return true;
+
+	wxChoice* pTheme = XRCCTRL(*this, "ID_THEME", wxChoice);
+
+	const int sel = pTheme->GetSelection();
+	const wxString theme = ((wxStringClientData*)pTheme->GetClientObject(sel))->GetData();
+
+	m_pOptions->SetOption(OPTION_THEME, theme);
+
+	wxNotebook *pPreview = XRCCTRL(*this, "ID_PREVIEW", wxNotebook);
+	wxString size = pPreview->GetPageText(pPreview->GetSelection());
+	m_pOptions->SetOption(OPTION_THEME_ICONSIZE, size);
+
+	return true;
+}
+
+bool COptionsPageThemes::Validate()
+{
+	return true;
+}
+
+bool COptionsPageThemes::DisplayTheme(const wxString& theme)
+{
+	wxString name, author, mail;
+	if (!CThemeProvider::GetThemeData(theme, name, author, mail))
+		return false;
+	if (name == _T(""))
+		return false;
+
+	if (author == _T(""))
+		author = _("N/a");
+	if (mail == _T(""))
+		mail = _("N/a");
+
+	bool failure = false;
+	SetStaticText(XRCID("ID_AUTHOR"), author, failure);
+	SetStaticText(XRCID("ID_EMAIL"), mail, failure);
+
+	wxNotebook *pPreview = XRCCTRL(*this, "ID_PREVIEW", wxNotebook);
+
+	wxString selected_size;
+	int selected_page = pPreview->GetSelection();
+	if (selected_page != -1)
+		selected_size = pPreview->GetPageText(selected_page);
+	else
+		selected_size = COptions::Get()->GetOption(OPTION_THEME_ICONSIZE);
+
+	pPreview->DeleteAllPages();
+
+	std::list<wxString> sizes = CThemeProvider::GetThemeSizes(theme);
+	for (std::list<wxString>::const_iterator iter = sizes.begin(); iter != sizes.end(); iter++)
+	{
+		const wxString& size = *iter;
+
+		int pos = size.Find('x');
+		if (pos <= 0 || pos >= (int)size.Len() - 1)
+			continue;
+
+        long width = 0;
+		long height = 0;
+		if (!size.Left(pos).ToLong(&width) ||
+			!size.Mid(pos + 1).ToLong(&height) ||
+			width <= 0 || height <= 0)
+		{
+			continue;
+		}
+
+		wxSize iconSize(width, height);
+
+		wxPanel* pPanel = new wxPanel();
+#ifdef __WXMSW__
+		// Drawing glitch on MSW
+		pPanel->Hide();
+#endif
+		pPanel->Create(pPreview);
+		wxSizer* pSizer = new wxBoxSizer(wxVERTICAL);
+		pPanel->SetSizer(pSizer);
+		pSizer->Add(new wxStaticText(pPanel, wxID_ANY, _("Preview:")), 0, wxLEFT|wxRIGHT|wxTOP, 5);
+
+		CIconPreview *pIconPreview = new CIconPreview(pPanel);
+		pIconPreview->LoadIcons(theme, iconSize);
+
+		pSizer->Add(pIconPreview, 1, wxGROW | wxALL, 5);
+
+		pPreview->AddPage(pPanel, size, selected_size == size);
+	}
+
+	return !failure;
+}
+
+void COptionsPageThemes::OnThemeChange(wxCommandEvent& event)
+{
+	wxChoice* pTheme = XRCCTRL(*this, "ID_THEME", wxChoice);
+
+	const int sel = pTheme->GetSelection();
+	const wxString theme = ((wxStringClientData*)pTheme->GetClientObject(sel))->GetData();
+
+	DisplayTheme(theme);
+}
+
+bool COptionsPageThemes::OnDisplayedFirstTime()
 {
 	bool failure = false;
 
@@ -172,98 +280,3 @@ bool COptionsPageThemes::LoadPage()
 
 	return !failure;
 }
-
-bool COptionsPageThemes::SavePage()
-{
-	wxChoice* pTheme = XRCCTRL(*this, "ID_THEME", wxChoice);
-
-	const int sel = pTheme->GetSelection();
-	const wxString theme = ((wxStringClientData*)pTheme->GetClientObject(sel))->GetData();
-
-	m_pOptions->SetOption(OPTION_THEME, theme);
-
-	wxNotebook *pPreview = XRCCTRL(*this, "ID_PREVIEW", wxNotebook);
-	wxString size = pPreview->GetPageText(pPreview->GetSelection());
-	m_pOptions->SetOption(OPTION_THEME_ICONSIZE, size);
-
-	return true;
-}
-
-bool COptionsPageThemes::Validate()
-{
-	return true;
-}
-
-bool COptionsPageThemes::DisplayTheme(const wxString& theme)
-{
-	wxString name, author, mail;
-	if (!CThemeProvider::GetThemeData(theme, name, author, mail))
-		return false;
-	if (name == _T(""))
-		return false;
-
-	if (author == _T(""))
-		author = _("N/a");
-	if (mail == _T(""))
-		mail = _("N/a");
-
-	bool failure = false;
-	SetStaticText(XRCID("ID_AUTHOR"), author, failure);
-	SetStaticText(XRCID("ID_EMAIL"), mail, failure);
-
-	wxNotebook *pPreview = XRCCTRL(*this, "ID_PREVIEW", wxNotebook);
-	pPreview->DeleteAllPages();
-
-	wxString selected = COptions::Get()->GetOption(OPTION_THEME_ICONSIZE);
-
-	std::list<wxString> sizes = CThemeProvider::GetThemeSizes(theme);
-	for (std::list<wxString>::const_iterator iter = sizes.begin(); iter != sizes.end(); iter++)
-	{
-		const wxString& size = *iter;
-
-		int pos = size.Find('x');
-		if (pos <= 0 || pos >= (int)size.Len() - 1)
-			continue;
-
-        long width = 0;
-		long height = 0;
-		if (!size.Left(pos).ToLong(&width) ||
-			!size.Mid(pos + 1).ToLong(&height) ||
-			width <= 0 || height <= 0)
-		{
-			continue;
-		}
-
-		wxSize iconSize(width, height);
-
-		wxPanel* pPanel = new wxPanel();
-#ifdef __WXMSW__
-		// Drawing glitch on MSW
-		pPanel->Hide();
-#endif
-		pPanel->Create(pPreview);
-		wxSizer* pSizer = new wxBoxSizer(wxVERTICAL);
-		pPanel->SetSizer(pSizer);
-		pSizer->Add(new wxStaticText(pPanel, wxID_ANY, _("Preview:")), 0, wxLEFT|wxRIGHT|wxTOP, 5);
-
-		CIconPreview *pIconPreview = new CIconPreview(pPanel);
-		pIconPreview->LoadIcons(theme, iconSize);
-
-		pSizer->Add(pIconPreview, 1, wxGROW | wxALL, 5);
-
-		pPreview->AddPage(pPanel, size, selected == size);
-	}
-
-
-	return !failure;
-}
-
-void COptionsPageThemes::OnThemeChange(wxCommandEvent& event)
-{
-	wxChoice* pTheme = XRCCTRL(*this, "ID_THEME", wxChoice);
-
-	const int sel = pTheme->GetSelection();
-	const wxString theme = ((wxStringClientData*)pTheme->GetClientObject(sel))->GetData();
-
-	DisplayTheme(theme);
-}
