@@ -4,10 +4,14 @@
 #include "chmoddialog.h"
 #include "filter.h"
 #include "queue.h"
+#include "local_filesys.h"
 
 CRecursiveOperation::CNewDir::CNewDir()
 {
 	recurse = true;
+	second_try = false;
+	link = false;
+	doVisit = true;
 }
 
 CRecursiveOperation::CRecursiveOperation(CState* pState)
@@ -69,20 +73,22 @@ void CRecursiveOperation::StartRecursiveOperation(enum OperationMode mode, const
 	NextOperation();
 }
 
-void CRecursiveOperation::AddDirectoryToVisit(const CServerPath& path, const wxString& subdir, const wxString& localDir /*=_T("")*/)
+void CRecursiveOperation::AddDirectoryToVisit(const CServerPath& path, const wxString& subdir, const wxString& localDir /*=_T("")*/, bool is_link /*=false*/)
 {
 	CNewDir dirToVisit;
-	dirToVisit.doVisit = true;
+
 	dirToVisit.localDir = localDir;
+	if (localDir != _T("") && localDir.Last() != CLocalFileSystem::path_separator)
+		dirToVisit.localDir += CLocalFileSystem::path_separator;
 	dirToVisit.parent = path;
 	dirToVisit.subdir = subdir;
+	dirToVisit.link = is_link;
 	m_dirsToVisit.push_back(dirToVisit);
 }
 
 void CRecursiveOperation::AddDirectoryToVisitRestricted(const CServerPath& path, const wxString& restrict, bool recurse)
 {
 	CNewDir dirToVisit;
-	dirToVisit.doVisit = true;
 	dirToVisit.parent = path;
 	dirToVisit.recurse = recurse;
 	dirToVisit.restrict = restrict;
@@ -104,7 +110,8 @@ bool CRecursiveOperation::NextOperation()
 			continue;
 		}
 
-		m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(dirToVisit.parent, dirToVisit.subdir));
+		CListCommand* cmd = new CListCommand(dirToVisit.parent, dirToVisit.subdir, dirToVisit.link ? LIST_FLAG_LINK : 0);
+		m_pState->m_pCommandQueue->ProcessCommand(cmd);
 		return true;
 	}
 
@@ -162,31 +169,33 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 		m_dirsToVisit.push_front(dir2);
 	}
 
-	// Check if we have already visited the directory
-	for (std::list<CServerPath>::const_iterator iter = m_visitedDirs.begin(); iter != m_visitedDirs.end(); iter++)
+	if (dir.link && !dir.recurse)
 	{
-		if (*iter == pDirectoryListing->path)
-		{
-			NextOperation();
-			return;
-		}
+		NextOperation();
+		return;
+	}
+
+	// Check if we have already visited the directory
+	if (!m_visitedDirs.insert(pDirectoryListing->path).second)
+	{
+		NextOperation();
+		return;
 	}
-	m_visitedDirs.push_back(pDirectoryListing->path);
 
 	const CServer* pServer = m_pState->GetServer();
 	wxASSERT(pServer);
 
 	if (!pDirectoryListing->GetCount())
 	{
-		wxFileName fn(dir.localDir, _T(""));
 		if (m_operationMode == recursive_download)
 		{
+			wxFileName fn(dir.localDir, _T(""));
 			wxFileName::Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
 			m_pState->RefreshLocalFile(fn.GetFullPath());
 		}
 		else if (m_operationMode == recursive_addtoqueue)
 		{
-			m_pQueue->QueueFile(true, true, fn.GetFullPath(), _T(""), CServerPath(), *pServer, -1);
+			m_pQueue->QueueFile(true, true, dir.localDir, _T(""), CServerPath(), *pServer, -1);
 			m_pQueue->QueueFile_Finish(false);
 		}
 	}
@@ -214,17 +223,19 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 		else if (filter.FilenameFiltered(entry.name, path, entry.dir, entry.size, false, 0))
 			continue;
 
-		if (entry.dir && !entry.link)
+		if (entry.dir && (!entry.link || m_operationMode != recursive_delete))
 		{
 			if (dir.recurse)
 			{
 				CNewDir dirToVisit;
-				wxFileName fn(dir.localDir, _T(""));
-				fn.AppendDir(entry.name);
 				dirToVisit.parent = pDirectoryListing->path;
 				dirToVisit.subdir = entry.name;
-				dirToVisit.localDir = fn.GetFullPath();
-				dirToVisit.doVisit = true;
+				dirToVisit.localDir = dir.localDir + entry.name + CLocalFileSystem::path_separator;
+				if (entry.link)
+				{
+					dirToVisit.link = true;
+					dirToVisit.recurse = false;
+				}
 				m_dirsToVisit.push_front(dirToVisit);
 			}
 		}
@@ -235,8 +246,9 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 			case recursive_addtoqueue:
 			case recursive_download:
 				{
-					wxFileName fn(dir.localDir, entry.name);
-					m_pQueue->QueueFile(m_operationMode == recursive_addtoqueue, true, fn.GetFullPath(), entry.name, pDirectoryListing->path, *pServer, entry.size);
+					m_pQueue->QueueFile(m_operationMode == recursive_addtoqueue, true,
+						dir.localDir + entry.name,
+						entry.name, pDirectoryListing->path, *pServer, entry.size);
 					added = true;
 				}
 				break;
@@ -295,7 +307,7 @@ void CRecursiveOperation::StopRecursiveOperation()
 	}
 }
 
-void CRecursiveOperation::ListingFailed()
+void CRecursiveOperation::ListingFailed(int error)
 {
 	if (m_operationMode == recursive_none)
 		return;
@@ -304,7 +316,15 @@ void CRecursiveOperation::ListingFailed()
 	if (m_dirsToVisit.empty())
 		return;
 
+	CNewDir dir = m_dirsToVisit.front();
 	m_dirsToVisit.pop_front();
+	if ((error & FZ_REPLY_CRITICALERROR) != FZ_REPLY_CRITICALERROR && !dir.second_try)
+	{
+		// Retry, could have been a temporary socket creating failure
+		// (e.g. hitting a blocked port) or a disconnect (e.g. no-filetransfer-timeout)
+		dir.second_try = true;
+		m_dirsToVisit.push_front(dir);
+	}
 
 	NextOperation();
 }
@@ -323,3 +343,31 @@ bool CRecursiveOperation::ChangeOperationMode(enum OperationMode mode)
 
 	return true;
 }
+
+void CRecursiveOperation::LinkIsNotDir()
+{
+	if (m_operationMode == recursive_none)
+		return;
+
+	wxASSERT(!m_dirsToVisit.empty());
+	if (m_dirsToVisit.empty())
+		return;
+
+	CNewDir dir = m_dirsToVisit.front();
+	m_dirsToVisit.pop_front();
+
+	const CServer* pServer = m_pState->GetServer();
+	if (!pServer)
+	{
+		NextOperation();
+		return;
+	}
+
+	wxString local_file = dir.localDir;
+	if (local_file.Last() == CLocalFileSystem::path_separator)
+		local_file.RemoveLast();
+	m_pQueue->QueueFile(m_operationMode == recursive_addtoqueue, true, local_file, dir.subdir, dir.parent, *pServer, -1);
+	m_pQueue->QueueFile_Finish(m_operationMode != recursive_addtoqueue);
+
+	NextOperation();
+}
