@@ -289,6 +289,8 @@ CSiteManager::CSiteManager()
 
 	m_pNotebook_Site = 0;
 	m_pNotebook_Bookmark = 0;
+
+	m_is_deleting = false;
 }
 
 CSiteManager::~CSiteManager()
@@ -302,7 +304,7 @@ CSiteManager::~CSiteManager()
 	}
 }
 
-bool CSiteManager::Create(wxWindow* parent, const CServer* pServer /*=0*/)
+bool CSiteManager::Create(wxWindow* parent, const wxString& connected_site_path, const CServer* pServer /*=0*/)
 {
 	m_pSiteManagerMutex = new CInterProcessMutex(MUTEX_SITEMANAGERGLOBAL, false);
 	if (!m_pSiteManagerMutex->TryLock())
@@ -391,10 +393,58 @@ bool CSiteManager::Create(wxWindow* parent, const CServer* pServer /*=0*/)
 
 	if (pServer)
 		CopyAddServer(*pServer);
+	else
+		MarkConnectedSite(connected_site_path);
 
 	return true;
 }
 
+void CSiteManager::MarkConnectedSite(wxString connected_site_path)
+{
+	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
+	if (!pTree)
+		return;
+
+	if (connected_site_path.Left(1) == _T("1"))
+	{
+		m_changed_bookmark_path = connected_site_path;
+		return;
+	}
+
+	if (connected_site_path.Left(1) != _T("0"))
+		return;
+
+	std::list<wxString> segments;
+	if (!UnescapeSitePath(connected_site_path.Mid(1), segments))
+		return;
+
+	wxTreeItemId current = m_ownSites;
+	while (!segments.empty())
+	{
+		wxTreeItemIdValue c;
+		wxTreeItemId child = pTree->GetFirstChild(current, c);
+		while (child)
+		{
+			if (pTree->GetItemText(child) == segments.front())
+				break;
+
+			child = pTree->GetNextChild(current, c);
+		}
+		if (!child)
+			return;
+
+		segments.pop_front();
+		current = child;
+	}
+
+	CSiteManagerItemData* data = reinterpret_cast<CSiteManagerItemData* >(pTree->GetItemData(current));
+	if (!data || data->m_type != CSiteManagerItemData::SITE)
+		return;
+
+	CSiteManagerItemData_Site *site_data = reinterpret_cast<CSiteManagerItemData_Site* >(data);
+	site_data->is_connected_item = true;
+}
+
 void CSiteManager::CreateControls(wxWindow* parent)
 {
 	wxXmlResource::Get()->LoadDialog(this, parent, _T("ID_SITEMANAGER"));
@@ -628,7 +678,7 @@ bool CSiteManager::Load()
 	TiXmlElement* pDocument = file.Load();
 	if (!pDocument)
 	{
-		wxString msg = wxString::Format(_("Could not load \"%s\", please make sure the file is valid and can be accessed.\nAny changes made in the Site Manager will not be saved."), file.GetFileName().GetFullPath().c_str());
+		wxString msg = file.GetError() + _T("\n") + _("Any changes made in the Site Manager will not be saved unless you repair the file.");
 		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
@@ -770,10 +820,12 @@ bool CSiteManager::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId /*=wx
 		CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
 		wxFileName file(wxGetApp().GetSettingsDir(), _T("sitemanager.xml"));
-		TiXmlElement* pDocument = GetXmlFile(file);
+		CXmlFile xml(file);
+
+		TiXmlElement* pDocument = xml.Load();
 		if (!pDocument)
 		{
-			wxString msg = wxString::Format(_("Could not load \"%s\", please make sure the file is valid and can be accessed.\nAny changes made in the Site Manager could not be saved."), file.GetFullPath().c_str());
+			wxString msg = xml.GetError() + _T("\n") + _("Any changes made in the Site Manager could not be saved.");
 			wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
 
 			return false;
@@ -788,22 +840,17 @@ bool CSiteManager::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId /*=wx
 		pElement = pDocument->InsertEndChild(TiXmlElement("Servers"))->ToElement();
 
 		if (!pElement)
-		{
-			delete pDocument->GetDocument();
-
 			return true;
-		}
 
 		bool res = Save(pElement, m_ownSites);
 
 		wxString error;
-		if (!SaveXmlFile(file, pDocument, &error))
+		if (!xml.Save(&error))
 		{
 			wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFullPath().c_str(), error.c_str());
 			wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
 		}
 
-		delete pDocument->GetDocument();
 		return res;
 	}
 
@@ -858,6 +905,12 @@ bool CSiteManager::SaveChild(TiXmlElement *pElement, wxTreeItemId child)
 		pNode->InsertEndChild(TiXmlText(utf8));
 
 		Save(pNode, child);
+
+		if (site_data->is_connected_item)
+		{
+			m_changed_bookmark_path = GetSitePath(child);
+			m_changed_bookmark_server = site_data->m_server;
+		}
 	}
 	else
 	{
@@ -1027,8 +1080,10 @@ bool CSiteManager::Verify()
 		const wxString remotePathRaw = XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->GetValue();
 		if (remotePathRaw != _T(""))
 		{
+			const wxString serverType = XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->GetStringSelection();
+
 			CServerPath remotePath;
-			remotePath.SetType(server.GetType());
+			remotePath.SetType(CServer::GetServerTypeFromName(serverType));
 			if (!remotePath.SetPath(remotePathRaw))
 			{
 				XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->SetFocus();
@@ -1184,12 +1239,19 @@ void CSiteManager::OnDelete(wxCommandEvent& event)
 	if (pTree->GetChildrenCount(parent) == 1)
 		pTree->Collapse(parent);
 
+	m_is_deleting = true;
+
 	pTree->Delete(item);
 	pTree->SelectItem(parent);
+
+	m_is_deleting = false;
 }
 
 void CSiteManager::OnSelChanging(wxTreeEvent& event)
 {
+	if (m_is_deleting)
+		return;
+
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
 		return;
@@ -1705,6 +1767,7 @@ void CSiteManager::OnCopySite(wxCommandEvent& event)
 	if (data->m_type == CSiteManagerItemData::SITE)
 	{
 		CSiteManagerItemData_Site* newData = new CSiteManagerItemData_Site(*(CSiteManagerItemData_Site *)data);
+		newData->is_connected_item = false;
 		newItem = pTree->AppendItem(parent, newName, 2, 2, newData);
 
 		wxTreeItemIdValue cookie;
@@ -1931,8 +1994,7 @@ wxMenu* CSiteManager::GetSitesMenu()
 	TiXmlElement* pDocument = file.Load();
 	if (!pDocument)
 	{
-		wxString msg = wxString::Format(_("Could not load \"%s\", please make sure the file is valid and can be accessed.\nAny changes made in the Site Manager will not be saved."), file.GetFileName().GetFullPath().c_str());
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		if (!predefinedSites)
 			return predefinedSites;
@@ -2180,6 +2242,7 @@ bool CSiteManager::MoveItems(wxTreeItemId source, wxTreeItemId target, bool copy
 		else if (data->m_type == CSiteManagerItemData::SITE)
 		{
 			CSiteManagerItemData_Site* newData = new CSiteManagerItemData_Site(*(CSiteManagerItemData_Site *)data);
+			newData->is_connected_item = false;
 			pTree->SetItemData(newItem, newData);
 		}
 		else
@@ -2241,7 +2304,7 @@ void CSiteManager::CopyAddServer(const CServer& server)
 	if (!Verify())
 		return;
 
-	AddNewSite(m_ownSites, server);
+	AddNewSite(m_ownSites, server, true);
 }
 
 wxString CSiteManager::FindFirstFreeName(const wxTreeItemId &parent, const wxString& name)
@@ -2278,7 +2341,7 @@ wxString CSiteManager::FindFirstFreeName(const wxTreeItemId &parent, const wxStr
 	return newName;
 }
 
-void CSiteManager::AddNewSite(wxTreeItemId parent, const CServer& server)
+void CSiteManager::AddNewSite(wxTreeItemId parent, const CServer& server, bool connected /*=false*/)
 {
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
@@ -2286,7 +2349,11 @@ void CSiteManager::AddNewSite(wxTreeItemId parent, const CServer& server)
 
 	wxString name = FindFirstFreeName(parent, _("New site"));
 
-	wxTreeItemId newItem = pTree->AppendItem(parent, name, 2, 2, new CSiteManagerItemData_Site(server));
+	CSiteManagerItemData_Site* pData = new CSiteManagerItemData_Site(server);
+	if (connected)
+		pData->is_connected_item = true;
+
+	wxTreeItemId newItem = pTree->AppendItem(parent, name, 2, 2, pData);
 	pTree->SortChildren(parent);
 	pTree->EnsureVisible(newItem);
 	pTree->SelectItem(newItem);
@@ -2425,8 +2492,7 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 
 	if (!pDocument)
 	{
-		wxString msg = wxString::Format(_("Could not load \"%s\", please make sure the file is valid and can be accessed.\nAny changes made in the Site Manager will not be saved."), file.GetFileName().GetFullPath().c_str());
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return 0;
 	}
@@ -2596,8 +2662,7 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 
 	if (!pDocument)
 	{
-		wxString msg = wxString::Format(_("Could not load \"%s\", please make sure the file is valid and can be accessed.\nAny changes made in the Site Manager will not be saved."), file.GetFileName().GetFullPath().c_str());
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
@@ -2644,17 +2709,12 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 	return true;
 }
 
-wxString CSiteManager::GetSitePath()
+wxString CSiteManager::GetSitePath(wxTreeItemId item)
 {
-	
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
 		return _T("");
 
-	wxTreeItemId item = pTree->GetSelection();
-	if (!item.IsOk())
-		return _T("");
-
 	CSiteManagerItemData* pData = reinterpret_cast<CSiteManagerItemData* >(pTree->GetItemData(item));
 	if (!pData)
 		return _T("");
@@ -2683,6 +2743,19 @@ wxString CSiteManager::GetSitePath()
 	return _T("0/") + path;
 }
 
+wxString CSiteManager::GetSitePath()
+{
+	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
+	if (!pTree)
+		return _T("");
+
+	wxTreeItemId item = pTree->GetSelection();
+	if (!item.IsOk())
+		return _T("");
+
+	return GetSitePath(item);
+}
+
 wxString CSiteManager::AddServer(CServer server)
 {
 	// We have to synchronize access to sitemanager.xml so that multiple processed don't write
@@ -2694,7 +2767,7 @@ wxString CSiteManager::AddServer(CServer server)
 
 	if (!pDocument)
 	{
-		wxString msg = wxString::Format(_("Could not load \"%s\", please make sure the file is valid and can be accessed.\nAny changes made in the Site Manager will not be saved."), file.GetFileName().GetFullPath().c_str());
+		wxString msg = file.GetError() + _T("\n") + _("The server could not be added.");
 		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return _T("");
@@ -2811,7 +2884,7 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 
 	if (!pDocument)
 	{
-		wxString msg = wxString::Format(_("Could not load \"%s\", please make sure the file is valid and can be accessed.\nAny changes made in the Site Manager will not be saved."), file.GetFileName().GetFullPath().c_str());
+		wxString msg = file.GetError() + _T("\n") + _("The bookmark could not be added.");
 		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
@@ -2887,7 +2960,7 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 
 	if (!pDocument)
 	{
-		wxString msg = wxString::Format(_("Could not load \"%s\", please make sure the file is valid and can be accessed.\nAny changes made in the Site Manager will not be saved."), file.GetFileName().GetFullPath().c_str());
+		wxString msg = file.GetError() + _T("\n") + _("The bookmarks could not be cleared.");
 		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
@@ -2927,3 +3000,11 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 
 	return true;
 }
+
+wxString CSiteManager::GetChangedBookmarkPath(const CServer* pServer)
+{
+	if (pServer && m_changed_bookmark_server != *pServer)
+		return wxEmptyString;
+	
+	return m_changed_bookmark_path;
+}
