@@ -59,6 +59,15 @@ bool CState::SetLocalDir(const wxString& dir, wxString *error /*=0*/)
 	}
 
 	CLocalPath p(m_localDir);
+#ifdef __WXMSW__
+	if (dir == _T("..") && !p.HasParent() && p.HasLogicalParent())
+	{
+		// Parent of C:\ is drive list
+		if (!p.MakeParent())
+			return false;
+	}
+	else
+#endif
 	if (!p.ChangePath(dir))
 		return false;
 
@@ -238,12 +247,33 @@ void CState::RefreshLocalFile(wxString file)
 		path.MakeParent(&file_name);
 		wxASSERT(!file_name.empty());
 	}
+
 	if (path != m_localDir)
 		return;
 
 	NotifyHandlers(STATECHANGE_LOCAL_REFRESH_FILE, file_name);
 }
 
+void CState::LocalDirCreated(const CLocalPath& path)
+{
+	if (!path.IsSubdirOf(m_localDir))
+		return;
+
+	wxString next_segment = path.GetPath().Mid(m_localDir.GetPath().Len());
+	int pos = next_segment.Find(CLocalPath::path_separator);
+	if (pos <= 0)
+	{
+		// Shouldn't ever come true
+		return;
+	}
+
+	// Current local path is /foo/
+	// Called with /foo/bar/baz/
+	// -> Refresh /foo/bar/
+	next_segment = next_segment.Left(pos);
+	NotifyHandlers(STATECHANGE_LOCAL_REFRESH_FILE, next_segment);
+}
+
 void CState::SetServer(const CServer* server)
 {
 	if (m_pServer)
@@ -736,7 +766,7 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 		}
 		else if (!IsRemoteIdle())
 		{
-			wxString msg(_("Another remote operation is already progress, cannot change directory now"));
+			wxString msg(_("Another remote operation is already in progress, cannot change directory now."));
 			wxMessageBox(msg, _("Synchronized browsing"), wxICON_EXCLAMATION);
 			return false;
 		}
