@@ -19,15 +19,21 @@ int CStatusLineCtrl::m_fieldOffsets[4];
 wxCoord CStatusLineCtrl::m_textHeight;
 bool CStatusLineCtrl::m_initialized = false;
 
-#define PROGRESSBAR_WIDTH 100
+#define PROGRESSBAR_WIDTH 102
 
 CStatusLineCtrl::CStatusLineCtrl(CQueueView* pParent, const t_EngineData* const pEngineData, const wxRect& initialPosition)
 	: m_pEngineData(pEngineData)
 {
 	wxASSERT(pEngineData);
 
+#ifdef __WXMSW__
 	Create(pParent, wxID_ANY, initialPosition.GetPosition(), initialPosition.GetSize());
+#else
+	Create(pParent->GetMainWindow(), wxID_ANY, initialPosition.GetPosition(), initialPosition.GetSize());
+#endif
+
 	SetOwnFont(pParent->GetFont());
+	SetForegroundColour(pParent->GetForegroundColour());
 	SetBackgroundStyle(wxBG_STYLE_CUSTOM);
 	SetBackgroundColour(pParent->GetBackgroundColour());
 
@@ -76,17 +82,30 @@ wxString FormatBytes(const wxLongLong& size)
 	if (!COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP))
 		return size.ToString();
 
+	static wxString sep;
+	static bool separator_initialized = false;
+	if (!separator_initialized)
+	{
+		separator_initialized = true;
 #ifdef __WXMSW__
-	wxChar sep[5];
-	int count = ::GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_STHOUSAND, sep, 5);
-	if (!count)
-		return size.ToString();
+		wxChar tmp[5];
+		int count = ::GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_STHOUSAND, tmp, 5);
+		if (count)
+			sep = tmp;
 #else
-	char* chr = nl_langinfo(THOUSEP);
-	if (!chr || !*chr)
-		return size.ToString();
-	wxChar sep = chr[0];
+		char* chr = nl_langinfo(THOUSEP);
+		if (chr && *chr)
+		{
+#if wxUSE_UNICODE
+			sep = wxString(chr, wxConvLibc);
+#else
+			sep = chr;
 #endif
+		}
+#endif
+	}
+	if (sep.empty())
+		return size.ToString();
 
 	wxString tmp = size.ToString();
 	const int len = tmp.Len();
@@ -114,7 +133,7 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent& event)
 	dc.SetFont(GetFont());
 	dc.SetPen(GetBackgroundColour());
 	dc.SetBrush(GetBackgroundColour());
-	dc.SetTextForeground(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
+	dc.SetTextForeground(GetForegroundColour());//wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
 
 	// Get character height so that we can center the text vertically.
 	wxCoord h = (rect.GetHeight() - m_textHeight) / 2;
@@ -266,8 +285,8 @@ void CStatusLineCtrl::DrawProgressBar(wxDC& dc, int x, int y, int height)
 		return;
 
 	int barSplit = wxLongLong(m_pStatus->currentOffset * (PROGRESSBAR_WIDTH - 2) / m_pStatus->totalSize).GetLo();
-	if (barSplit > PROGRESSBAR_WIDTH)
-		barSplit = PROGRESSBAR_WIDTH;
+	if (barSplit > (PROGRESSBAR_WIDTH - 2))
+		barSplit = PROGRESSBAR_WIDTH - 2;
 
 	// Draw right part
 	dc.SetPen(*wxTRANSPARENT_PEN);
