@@ -36,8 +36,13 @@ protected:
 
 	void SetTransferStatus(const CTransferStatus* pStatus);
 
+	void PrepareUpdateCheckPage();
 	void StartUpdateCheck();
 
+	int SendTransferCommand();
+
+	wxString GetDownloadDir();
+
 	DECLARE_EVENT_TABLE()
 	void OnCheck(wxCommandEvent& event);
 	void OnPageChanging(wxWizardEvent& event);
@@ -59,6 +64,9 @@ protected:
 
 	bool m_loaded;
 
+	// Hold changelog of what's new
+	wxString m_news;
+
 	// Auto check related functions and variables
 	// ------------------------------------------
 
@@ -68,6 +76,7 @@ protected:
 
 	bool m_autoUpdateCheckRunning;
 	bool m_updateShown;
+	bool m_start_check;
 };
 
 #endif //FZ_MANUALUPDATECHECK
