@@ -14,6 +14,10 @@ CWindowStateManager::CWindowStateManager(wxTopLevelWindow* pWindow)
 
 	m_pWindow->Connect(wxID_ANY, wxEVT_SIZE, wxSizeEventHandler(CWindowStateManager::OnSize), 0, this);
 	m_pWindow->Connect(wxID_ANY, wxEVT_MOVE, wxMoveEventHandler(CWindowStateManager::OnMove), 0, this);
+
+#ifdef __WXGTK__
+	m_maximize_requested = 0;
+#endif
 }
 
 CWindowStateManager::~CWindowStateManager()
@@ -104,7 +108,14 @@ bool CWindowStateManager::Restore(unsigned int optionId)
 		// doesn't fully set the height properly at least under wxMSW
 		m_pWindow->SetClientSize(client_width, client_height);
 
+#ifdef __WXMSW__
+		m_pWindow->Show();
+#endif
 		m_pWindow->Maximize();
+#ifdef __WXGTK__
+		if (!m_pWindow->IsMaximized())
+			m_maximize_requested = 4;
+#endif //__WXGTK__
 	}
 	else
 	{
@@ -124,6 +135,10 @@ bool CWindowStateManager::Restore(unsigned int optionId)
 
 void CWindowStateManager::OnSize(wxSizeEvent& event)
 {
+#ifdef __WXGTK__
+	if (m_maximize_requested)
+		m_maximize_requested--;
+#endif
 	if (!m_pWindow->IsIconized())
 	{
 		m_lastMaximized = m_pWindow->IsMaximized();
