@@ -11,13 +11,17 @@ public:
 
 	static void ClearCache();
 
+	enum wrap_result {
+		wrap_failed = 0x01,
+		wrap_didwrap = 0x02
+	};
 	// Find all wxStaticText controls in the given window(s) and wrap them, so
 	// that the window has the right aspect ratio...
 	bool WrapRecursive(wxWindow* wnd, double ratio, const char* name = "", wxSize canvas = wxSize(), wxSize minRequested = wxSize());
 	bool WrapRecursive(std::vector<wxWindow*>& windows, double ratio, const char* name = "", wxSize canvas = wxSize(), wxSize minRequested = wxSize());
 
 	// .. or does not exceed the given maximum length.
-	bool WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max);
+	int WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max);
 
 	// Find all wxStaticText controls in the window and unwrap them
 	bool UnwrapRecursive(wxWindow* wnd, wxSizer* sizer);
@@ -33,6 +37,8 @@ public:
 	void CheckLanguage();
 
 protected:
+	void UnwrapRecursive_Wrapped(const std::list<int> &wrapped, std::vector<wxWindow*> &windows, bool remove_fitting = false);
+
 	void SetWidthToCache(const char* name, int width);
 
 	std::map<wxChar, unsigned int> m_charWidths;
