@@ -3,6 +3,7 @@
 #include "filezillaapp.h"
 #include "Options.h"
 #include <wx/ffile.h>
+#include <wx/log.h>
 
 CXmlFile::CXmlFile(const wxString& fileName)
 {
@@ -52,9 +53,15 @@ TiXmlElement* CXmlFile::Load(const wxFileName& fileName)
 	delete m_pDocument;
 	m_pDocument = 0;
 
-	TiXmlElement* pElement = GetXmlFile(m_fileName);
+	wxString error;
+	TiXmlElement* pElement = GetXmlFile(m_fileName, true, &error);
 	if (!pElement)
 	{
+		m_error.Printf(_("The file '%s' could not be loaded."), m_fileName.GetFullPath().c_str());
+		if (!error.empty())
+			m_error += _T("\n") + error;
+		else
+			m_error += wxString(_T("\n")) + _("Make sure the file can be accessed and is a well-formed XML document.");
 		m_modificationTime = wxDateTime();
 		return 0;
 	}
@@ -389,35 +396,48 @@ bool GetTextElementBool(TiXmlElement* node, const char* name, bool defValue /*=f
 	}
 }
 
-TiXmlDocument* LoadXmlDocument(const wxString& file)
+bool LoadXmlDocument(TiXmlDocument* pXmlDocument, const wxString& file, wxString* error /*=0*/)
 {
 	wxFFile f(file, _T("rb"));
 	if (!f.IsOpened())
-		return 0;
+	{
+		if (error)
+		{
+			const wxChar* s = wxSysErrorMsg();
+			if (s && *s)
+				*error = s;
+			else
+				*error = _("Unknown error opening the file. Make sure the file can be accessed and is a well-formed XML document.");
+		}
+		return false;
+	}
 
-	TiXmlDocument* pXmlDocument = new TiXmlDocument();
-	pXmlDocument->SetCondenseWhiteSpace(false);
 	if (!pXmlDocument->LoadFile(f.fp()))
 	{
-		delete pXmlDocument;
-		return 0;
+		if (pXmlDocument->ErrorId() != TiXmlBase::TIXML_ERROR_DOCUMENT_EMPTY)
+		{
+			if (error)
+			{
+				const char* s = pXmlDocument->ErrorDesc();
+				error->Printf(_("The XML document is not well-formed: %s"), wxString(s, wxConvLibc).c_str());
+			}
+			return false;
+		}
 	}
-	return pXmlDocument;
+	return true;
 }
 
 // Opens the specified XML file if it exists or creates a new one otherwise.
 // Returns 0 on error.
-TiXmlElement* GetXmlFile(wxFileName file, bool create /*=true*/)
+TiXmlElement* GetXmlFile(wxFileName file, bool create /*=true*/, wxString* error /*=0*/)
 {
 	if (wxFileExists(file.GetFullPath()) && file.GetSize() > 0)
 	{
 		// File does exist, open it
 
-		TiXmlDocument* pXmlDocument = LoadXmlDocument(file.GetFullPath());
-		if (!pXmlDocument)
-			return 0;
-
-		if (!pXmlDocument->FirstChildElement("FileZilla3"))
+		TiXmlDocument* pXmlDocument = new TiXmlDocument;
+		pXmlDocument->SetCondenseWhiteSpace(false);
+		if (!LoadXmlDocument(pXmlDocument, file.GetFullPath(), error))
 		{
 			delete pXmlDocument;
 			return 0;
@@ -426,11 +446,19 @@ TiXmlElement* GetXmlFile(wxFileName file, bool create /*=true*/)
 		TiXmlElement* pElement = pXmlDocument->FirstChildElement("FileZilla3");
 		if (!pElement)
 		{
-			delete pXmlDocument;
-			return 0;
+			if (pXmlDocument->FirstChildElement())
+			{
+				// Not created by FileZilla3
+				delete pXmlDocument;
+
+				if (error)
+					*error = _("Unknown root element, the file does not appear to be generated by FileZilla.");
+				return 0;
+			}
+			pElement = pXmlDocument->InsertEndChild(TiXmlElement("FileZilla3"))->ToElement();
 		}
-		else
-			return pElement;
+
+		return pElement;
 	}
 	else
 	{
