@@ -1387,9 +1387,9 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(),
 				false
-		},
-		MVS
-	});
+			},
+			MVS
+		});
 
 	// Migrated MVS file
 	m_entries.push_back((t_entry){
@@ -1406,9 +1406,9 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(),
 				false
-		},
-		MVS
-	});
+			},
+			MVS
+		});
 
 	// z/VM, another IBM abomination. Description by Alexandre Charbey
 	// Requires type set to ZVM or it cannot be parsed.
@@ -1442,9 +1442,69 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(4, wxDateTime::Oct, 2005, 15, 28, 42),
 				false
-		},
-		ZVM
-	});
+			},
+			ZVM
+		});
+
+	m_entries.push_back((t_entry){
+			"drwxr-xr-x 3 slopri devlab 512 71-unix-dateless",
+			{
+				_T("71-unix-dateless"),
+				512,
+				_T("drwxr-xr-x"),
+				_T("slopri devlab"),
+				true,
+				false,
+				_T(""),
+				false,
+				false,
+				wxDateTime(),
+				false
+			},
+			DEFAULT
+		});
+
+	utc = wxDateTime(5, wxDateTime::Nov, 2008, 16, 52, 15);
+	utc.MakeFromTimezone(wxDateTime::UTC);
+	m_entries.push_back((t_entry){
+			"Type=file;mOdIfY=20081105165215;size=1234; 72-MLSD-file",
+			{
+				_T("72-MLSD-file"),
+				1234,
+				_T(""),
+				_T(""),
+				false,
+				false,
+				_T(""),
+				true,
+				true,
+				utc,
+				false
+			},
+			DEFAULT
+		});
+
+	// Yet another MVS format.
+	// Follows the below structure but with all but the first two and the last field empty.
+	// Furthermore, Unit is "Tape"
+	// Volume Unit    Referred Ext Used Recfm Lrecl BlkSz Dsorg Dsname
+	m_entries.push_back((t_entry) {
+			"V43525 Tape                                             73-MSV-TAPE.FILE",
+			{
+				_T("73-MSV-TAPE.FILE"),
+				-1,
+				_T(""),
+				_T(""),
+				false,
+				false,
+				_T(""),
+				false,
+				false,
+				wxDateTime(),
+				false
+			},
+			MVS
+		});
 
 /*
 	wxString name;
